/*
 * Program: FileMaker
 * 
 * Purpose: file manager to demonstrate a few new classes written 
 * for wxXt: wxTreeCtrl, wxDirTree, wxListCtrl, wxFileList, wxDragCanvas
 *
 * Author: Robert Roebling
 *
 * Copyright: (C) 1997, GNU (Robert Roebling)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef __GNUG__
#pragma implementation "FileMaker.h"
#endif

//#define UseMyToolBar

#define  Uses_wxApp
#define  Uses_wxWindow
#ifdef UseMyToolBar
#define  Uses_wxToolBar
#endif
#define  Uses_wxButton
#define  Uses_wxCheckBox
#define  Uses_wxChoice
#define  Uses_wxFontDialog
#define  Uses_wxFrame
#define  Uses_wxGauge
#define  Uses_wxListBox
#define  Uses_wxMessage
#define  Uses_wxMultiText
#define  Uses_wxPanel
#define  Uses_wxPostScriptDC
#define  Uses_wxRadioBox
#define  Uses_wxSlider
#define  Uses_wxText
#define  Uses_wxTextWindow
#define  Uses_wxTimer
#define  Uses_wxCanvas
#define  Uses_wxMenu
#define  Uses_wxMenuBar
#define  Uses_wxHashTable
#define  Uses_wxString
#define  Uses_wxMemoryDC
#define  Uses_XtIntrinsic
#include "wx.h"

#include "WIN_dirtree.h"
#include "WIN_filelist.h"
#include "WIN_toolbar3.h"
#include "WIN_rawtext.h"

#include "FileMaker.h"
#include "FMDialogs.h"

#include "pics/folder.xpm"
#include "pics/txt.xpm"
#include "pics/list.xpm"

#ifdef UseMyToolBar
#include "pics/exit.xpm"
#endif


const LIVE_EXIT         = 100;

const LIVE_COPY         = 110;
const LIVE_MOVE         = 111;
const LIVE_DELETE       = 112;

const LIVE_CREATE_SYM   = 113;
const LIVE_EDIT_SYM     = 114;

const LIVE_CHANGE_PERM  = 115;

const LIVE_CREATE_DIR   = 121;
const LIVE_RESCAN       = 120;

const LIVE_TREE         = 130;
const LIVE_SINGLE       = 131;
const LIVE_NORTON       = 132;

const LIVE_FILTER       = 133;
const LIVE_SORT         = 134;
const LIVE_SHOW_HIDDEN  = 135;

const LIVE_MARK         = 136;
const LIVE_UNMARK       = 137;

const LIVE_SHORT        = 140;
const LIVE_TABLE        = 141;
const LIVE_ICON         = 142;

const LIVE_FIND_FILE    = 150;
const LIVE_MOUNT        = 152;
const LIVE_START_TERM   = 153;

const LIVE_OPTIONS      = 161;
const LIVE_USERMENU     = 162;
const LIVE_EXT          = 163;

const LIVE_HELP         = 170;
const LIVE_ABOUT        = 171;

//-----------------------------------------------------------------------------
// local and exported data
//-----------------------------------------------------------------------------

#ifdef UseMyToolBar
static int tbar_pos = 0;
static int tbar_pos_circulate[] = 
  { wxALIGN_TOP, wxALIGN_LEFT, wxALIGN_RIGHT, wxALIGN_BOTTOM };
#endif

Bool             wxIsRoot = FALSE;
wxLiveFrame     *myFrame = NULL;
wxLivePanel     *myPanel = NULL;
wxFileCtrl      *myRightList = NULL;
wxFileCtrl      *myLeftList = NULL;
wxDirTree       *myTree = NULL;
//wxLivePage      *page;
wxBrush         *backgroundBrush;

enum displayType { displaySingle, displayTree, displayNorton };
displayType      myDisplay;

#ifdef UseMyToolBar
static wxToolBar *MakeToolBar( void );
#endif

static wxMenuBar *MakeMenuBar( void );

//-----------------------------------------------------------------------------
// main program
//-----------------------------------------------------------------------------

wxLiveApp TheApp;

//-----------------------------------------------------------------------------
// wxLiveApp 
//-----------------------------------------------------------------------------

wxFrame *wxLiveApp::OnInit(void)
{
  char buf[100]; 
  wxGetUserId( buf, 100 );
  wxIsRoot = strcmp( buf, "root" ) == 0;

  wxColour *bg = new wxColour( 208, 180, 176 );  // deallocate
  backgroundBrush = new wxBrush( (*bg), wxSOLID );

  int w = 600; 
  int h = 400;
  myFrame = new wxLiveFrame( NULL, "FileMaker - wxXt demo", 0, 0, w, h );
  myFrame->SetButtonFont( wxTheFontList->FindOrCreateFont( 12, wxROMAN, wxNORMAL, wxNORMAL ) );

#ifdef UseMyToolBar
  myFrame->SetToolBar( MakeToolBar(), tbar_pos_circulate[tbar_pos] );
#endif

  myFrame->SetMenuBar( MakeMenuBar() );
  myFrame->CreateStatusLine(1);

  imageList = new wxImageList( 64, 64 );
  wxBitmap *bm = new wxBitmap( folder_xpm );
  imageList->Add( bm );
  
  bm = new wxBitmap( txt_xpm );
  imageList->Add( bm );

  bm = new wxBitmap( list_xpm );
  
  int step = 40;
  int gap = 60;
  int x = 5;
  wxToolBar3 *toolBar = myFrame->NewToolBar3( wxTBTop );
  toolBar->AppendTool( bm, x, 3, LIVE_CREATE_SYM, FALSE, "Create symbolic link" );
  x += step;
  toolBar->AppendTool( bm, x, 3, LIVE_EDIT_SYM, FALSE, "Edit symbolic link" );
  
  x += step;
  toolBar->AppendTool( bm, x, 3, LIVE_CHANGE_PERM, FALSE, "Change permissions" );
  
  x += gap;        
  toolBar->AppendTool( bm, x, 3, LIVE_SHORT, FALSE, "Create directoy" );
  x += gap;        
  toolBar->AppendTool( bm, x, 3, LIVE_SHORT, FALSE, "List view" );
  x += step;
  toolBar->AppendTool( bm, x, 3, LIVE_TABLE, FALSE, "Report view" );
  
  x += step;
  toolBar->AppendTool( bm, x, 3, LIVE_ICON, FALSE, "Icon view" );
  
  x += gap;        
  toolBar->AppendTool( bm, x, 3, LIVE_TREE, FALSE, "Tree window" );
  x += step;
  toolBar->AppendTool( bm, x, 3, LIVE_NORTON, FALSE, "Commander window" );
  
  x += step;
  toolBar->AppendTool( bm, x, 3, LIVE_SINGLE, FALSE, "Single window" );
  x += gap;
  toolBar->AppendTool( bm, x, 3, LIVE_MOUNT, FALSE, "Mount devices" );
  x += step;
  toolBar->AppendTool( bm, x, 3, LIVE_FIND_FILE, FALSE, "Find files" );
  
  myDisplay = displayTree;

  myRightList = DEBUG_NEW wxFileCtrl( myFrame, 101, -1, -1, -1, "/", myFrame );
  myRightList->SetImageList( imageList, wxIMAGE_LIST_NORMAL );

  MakeDisplay();

//  myPanel = DEBUG_NEW wxLivePanel( myFrame, 101, -1, -1, -1);

  myFrame->Show( TRUE );

  return myFrame;
}

void wxLiveApp::MakeDisplay( void )
{
  switch (myDisplay)
  {
    case displaySingle:
    {
      if (myTree)
      {
        myTree->Show( FALSE );
        delete myTree;
        myTree = NULL;
      };
      if (myLeftList)
      {
        myLeftList->Show( FALSE );
        delete myLeftList;
        myLeftList = NULL;
      };
      break;
    };
    case displayTree:
    {
      if (myLeftList)
      {
        myLeftList->Show( FALSE );
        delete myLeftList;
        myLeftList = NULL;
      };
      if (!myTree)
      {
        myTree = DEBUG_NEW wxDirTree( myFrame, -1, -1, 100, -1 );
        myTree->AddToCurrent( new wxDirBranch( "/", myRightList ) );
        myTree->AddToCurrent( new wxDirBranch( "/home", myRightList ) );
        myTree->AddToCurrent( new wxDirBranch( "/mnt", myRightList ) );
        myTree->AddToCurrent( new wxDirBranch( "/proc", myRightList ) );
//      myTree->AddToCurrent( new wxLiveBranch( TRUE, "System", liveSYSTEM ) );
//      myTree->AddToCurrent( new wxLiveBranch( FALSE, "Kernel", liveKERNEL ) );
//      myTree->AddToCurrent( new wxLiveBranch( FALSE, "Options", liveOPTIONS ) );
//      myTree->AddToCurrent( new wxLiveBranch( FALSE, "X", liveX ) );
      };
      break;
    };
    case displayNorton:
    {
      if (myTree)
      {
        myTree->Show( FALSE );
        delete myTree;
        myTree = NULL;
      };
      if (!myLeftList)
      {
        myLeftList = DEBUG_NEW wxFileCtrl( myFrame, 101, -1, -1, -1, "/", myFrame );
        myLeftList->SetImageList( imageList, wxIMAGE_LIST_NORMAL );
	myLeftList->SetListStyleFlag( myRightList->RawGetMode() );
	myLeftList->Update();
	myLeftList->Refresh();
      };
      break;
    };
  };
};

int wxLiveApp::OnExit(void)
{
    return 0;
}

//-----------------------------------------------------------------------------
// wxLiveFrame 
//-----------------------------------------------------------------------------

IMPLEMENT_DYNAMIC_CLASS(wxLiveFrame,wxDecoratedFrame);
//IMPLEMENT_DYNAMIC_CLASS(wxLiveFrame,wxFrame);

void wxLiveFrame::OnMenuCommand( int id )
{
  switch (id) 
  {
    case LIVE_ABOUT:
    { 
      wxAboutDia( this );
      break;
    };
    case LIVE_START_TERM:
    {
      wxString s;
      myRightList->GetDir( s );
      wxSetWorkingDirectory( s.GetData() );
      wxShell();
      break;
    };
    case LIVE_EXIT:
    {
      OnClose();
      delete this;
      break;
    };
    case LIVE_SINGLE:
    {
      myDisplay = displaySingle;
      TheApp.MakeDisplay();
      DoResize();
      break;
     }; 
    case LIVE_NORTON:
    {
      myDisplay = displayNorton;
      TheApp.MakeDisplay();
      DoResize();
      break;
     }; 
    case LIVE_TREE:
    {
      myDisplay = displayTree;
      TheApp.MakeDisplay();
      DoResize();
      break;
     }; 
    case LIVE_SHORT:
    {
      if (myLeftList) myLeftList->ChangeToListMode();
      myRightList->ChangeToListMode();
      break;
    };
    case LIVE_TABLE:
    {
      if (myLeftList) myLeftList->ChangeToReportMode();
      myRightList->ChangeToReportMode();
      break;
    };
    case LIVE_ICON:
    {
      if (myLeftList) myLeftList->ChangeToIconMode();
      myRightList->ChangeToIconMode();
      break;
    };
    case LIVE_COPY:        DoCopy(); break;
    case LIVE_DELETE:      DoDelete(); break;
    case LIVE_MOVE:        DoMove(); break;
    case LIVE_EDIT_SYM:    DoEditSymlink(); break;
    case LIVE_CREATE_SYM:  DoCreateSymlink(); break;
    case LIVE_CREATE_DIR:  DoCreateDir(); break;
    case LIVE_CHANGE_PERM: DoChangePerm(); break;
    case LIVE_FIND_FILE:   DoFindFile(); break;
    case LIVE_MOUNT:       DoMount(); break;
  };
}

void wxLiveFrame::DoCopy( void )
{
};

void wxLiveFrame::DoDelete( void )
{
};

void wxLiveFrame::DoMove( void )
{
};

void wxLiveFrame::DoEditSymlink( void )
{
};

void wxLiveFrame::DoCreateSymlink( void )
{
};

void wxLiveFrame::DoCreateDir( void )
{
};

void wxLiveFrame::DoChangePerm( void )
{
};

void wxLiveFrame::DoFindFile( void )
{
  wxString s;
  myRightList->GetDir( s );
  wxFindDia( this, s );
};

void wxLiveFrame::DoMount( void )
{
  wxMountDia( this );
};

void wxLiveFrame::DoResize( void )
{
  int w = 0;
  int h = 0;
  GetSize( &w, &h );
  OnSize( w, h );
  return;
  switch (myDisplay)
  {
    case displaySingle:
    {
      if (myRightList) myRightList->SetSize( 0, 0, w, h );
      break;
    };
    case displayTree:
    {
      if (myTree) myTree->SetSize( 0, 0, w/3, h );
      if (myRightList) myRightList->SetSize( w/3, 0, w*2/3, h );
      break;
    };
    case displayNorton:
    {
      if (myLeftList) myLeftList->SetSize( 0, 0, w/2, h );
      if (myRightList) myRightList->SetSize( w/2, 0, w/2, h );
      break;
    };
  };
};

void wxLiveFrame::OnDecoratedSize( int x, int y, int w, int h )
{
  switch (myDisplay)
  {
    case displaySingle:
    {
      if (myRightList) myRightList->SetSize( x, y, w, h );
      break;
    };
    case displayTree:
    {
      if (myTree) myTree->SetSize( x, y, w/3, h );
      if (myRightList) myRightList->SetSize( w/3, y, w*2/3, h );
      break;
    };
    case displayNorton:
    {
      if (myLeftList) myLeftList->SetSize( x, y, w/2, h );
      if (myRightList) myRightList->SetSize( w/2, y, w/2, h );
      break;
    };
  };
};

Bool wxLiveFrame::OnClose( void )
{
  return TRUE;
};


//-----------------------------------------------------------------------------
// wxLivePanel
//-----------------------------------------------------------------------------

wxLivePanel::wxLivePanel( wxFrame *parent, int x, int y,
  int width, int height, int style, char *name ): 
  wxPanel( parent, x, y, width, height, style, name ) 
{
//  SetHorizontalMargin( 5 );
//  SetHorizontalSpacing( 5 );
//  SetVerticalMargin( 3 );   
//  SetVerticalSpacing( 3 );
//  page = new lpHome();
//  page->SetPanel( this );
//  myFrame->SetTitle( page->GetTitle() );
//  toolbar->MakeHomeTools();
//  page->MakeChildren();
   
  Fit();     
};

void wxLivePanel::OnCommand( wxWindow &WXUNUSED(win), wxCommandEvent &WXUNUSED(event) )
{
//  wxItem *item = (wxItem*)&win;
//  if (item == (wxItem*)rotate_tbar) 
//  {
//    tbar_pos = (tbar_pos + 1) % wxNumberOf(tbar_pos_circulate);
//    fflush(stdout);
//    myFrame->SetToolBar( myFrame->GetToolBar(), tbar_pos_circulate[tbar_pos] );
//    return;
//  };
//  page->OnCommand( item, event ); 
};

void wxLivePanel::OnDefaultAction( wxItem *WXUNUSED(item) )
{
//   page->OnDefaultAction( item );
};

//void wxLivePanel::ChangePage( wxLivePage* WXUNUSED(p) )
//{
//  Show( FALSE );
//  DestroyChildren();
//  cursor_x = 0;
//  cursor_y = 0;
//  line_extent = 0;
//  delete page;
//  page = p;
//  page->SetPanel( this );
//  page->MakeChildren();
//  page->ReadData();
//  page->ShowData();
//  Show( TRUE ); 
//};

//-----------------------------------------------------------------------------
// MenuBar stuff
//-----------------------------------------------------------------------------

static wxMenuBar *MakeMenuBar()
{
  wxMenuBar *menubar       = DEBUG_NEW wxMenuBar;
  wxMenu    *file_menu     = DEBUG_NEW wxMenu;
  wxMenu    *view_menu     = DEBUG_NEW wxMenu;
  wxMenu    *commands_menu = DEBUG_NEW wxMenu;
  wxMenu    *options_menu  = DEBUG_NEW wxMenu;
  wxMenu    *help_menu     = DEBUG_NEW wxMenu;

  file_menu->Append( LIVE_COPY, "&Copy...\tMeta-C", "Copy file(s)" );
  file_menu->Append( LIVE_MOVE, "&Move...\tMeta-M", "Move file(s)" );
  file_menu->Append( LIVE_DELETE, "&Delete...\tMeta-D", "Delete file(s)" );
  file_menu->AppendSeparator();
  file_menu->Append( LIVE_CREATE_SYM, "Create &sym link...\tMeta-S", "Create new symbolic link" );
  file_menu->Append( LIVE_EDIT_SYM, "&Edit sym link...\tMeta-E", "Edit symbolic link" );
  file_menu->AppendSeparator();
  file_menu->Append( LIVE_CHANGE_PERM, "Change &permissions...\tMeta-P", "Edit file permissions (chmod)" );
  file_menu->AppendSeparator();
  file_menu->Append( LIVE_CREATE_DIR, "Create &new dir...\tMeta-N", "Create new directory" );
  file_menu->AppendSeparator();
  file_menu->Append( LIVE_RESCAN, "&Rescan\tMeta-R", "Rescan directory" );
  file_menu->AppendSeparator();
  file_menu->Append( LIVE_EXIT, "&Exit...\tMeta-X", "Exit FileMaker" );

  view_menu->Append( LIVE_SINGLE, "Single &view\tCtrl-V", "Display only one list window" );
  view_menu->Append( LIVE_TREE, "&Tree view\tCtrl-T", "Display tree and list window" );
  view_menu->Append( LIVE_NORTON, "&Norton view\tCtrl-N", "Display two list windows" );
  view_menu->AppendSeparator();
  view_menu->Append( LIVE_SHORT, "Short &list\tCtrl-L", "Display only filenames" );
  view_menu->Append( LIVE_TABLE, "&Extended list\tCtrl-E", "Display filename plus info" );
  view_menu->Append( LIVE_ICON, "&Icon list\tCtrl-I", "Display filename plus icon" );
  view_menu->AppendSeparator();
  view_menu->Append( LIVE_FILTER, "&Filter...\tCtrl-F", "Use filter for display list" );
  view_menu->Append( LIVE_SORT, "Sort &criteria...\tCtrl-C", "Determine sort criteria" );
  view_menu->Append( LIVE_SHOW_HIDDEN, "Show &hidden\tCtrl-H", "Show hidden files" );
  view_menu->AppendSeparator();
  view_menu->Append( LIVE_MARK, "&Mark files...\tCtrl-M", "Choose mask to mark files" );
  view_menu->Append( LIVE_UNMARK, "&Unmark files...\tCtrl-U", "Choose mask to unmark files" );

  commands_menu->Append( LIVE_FIND_FILE, "&Find files...\tCtrl-F", "Find files" );
  commands_menu->Append( LIVE_MOUNT, "&Mount devices...\tCtrl-M", "Mount devices, such as floppies" );
  commands_menu->Append( LIVE_START_TERM, "Start &shell\tCtrl-S", "Start an empty terminal window" );

  options_menu->Append( LIVE_OPTIONS, "&Options...", "Various options" );
  options_menu->Append( LIVE_USERMENU, "&User commands...", "Edit user menu" );
  options_menu->Append( LIVE_EXT, "&Extensions...", "Edit extension and their bindings" );


  help_menu->Append( LIVE_ABOUT, "&About..\tMeta-A", "Information about FileMaker" );

  menubar->Append( file_menu, "&File" );
  menubar->Append( view_menu, "&View" );
  menubar->Append( commands_menu, "&Commands" );
  menubar->Append( options_menu, "&Options" );
  menubar->Append( help_menu, "&Help" );

  return menubar; 
};

#ifdef UseMyToolBar

//-----------------------------------------------------------------------------
// ToolBar stuff
//-----------------------------------------------------------------------------

class myToolBar: public wxToolBar
{
  public:

     myToolBar( wxFrame *f ) : wxToolBar( f ) {};
     virtual Bool OnLeftClick( int toolIndex, Bool toggled );
};

Bool myToolBar::OnLeftClick( int toolIndex, Bool toggled )
{
  if (toggled) myFrame->OnMenuCommand( toolIndex );
  return TRUE;
};

static wxToolBar *MakeToolBar()
{
  wxToolBar *tb = DEBUG_NEW myToolBar( myFrame );

#define REF(x) &(x ## _bm )

  wxBitmap exit_bm( exit_xpm );

  tb->SetMargins( 4, 4 );
  tb->AddTool( LIVE_EXIT, REF(exit) );

  return tb;

};

#endif

