/*
 * File:	ImagList.h
 * Purpose:	wxImageList
 * Author:	Julian Smart, Robert Roebling
 * Created:	1997
 * Copyright:   L-GPL
 */

#ifndef __IMAGLISTH__
#define __IMAGLISTH__

#ifdef __GNUG__
#pragma interface
#endif

/*
 * wxImageList is used for wxListCtrl, wxTreeCtrl. These controls refer to
 * images for their items by an index into an image list.
 * A wxImageList is capable of creating images with optional masks from
 * a variety of sources - a single bitmap plus a colour to indicate the mask,
 * two bitmaps, or an icon.
 *
 * Image lists can also create and draw images used for drag and drop functionality.
 * This is not yet implemented in wxImageList. We need to discuss a generic API
 * for doing drag and drop and see whether it ties in with the Win95 view of it.
 * See below for candidate functions and an explanation of how they might be
 * used.
 */

// Flags for Draw
#define wxIMAGELIST_DRAW_NORMAL         0x0001
#define wxIMAGELIST_DRAW_TRANSPARENT    0x0002
#define wxIMAGELIST_DRAW_SELECTED       0x0004
#define wxIMAGELIST_DRAW_FOCUSED        0x0008

class wxImageList: public wxObject
{
//  DECLARE_DYNAMIC_CLASS(wxImageList)
 protected:
 
    int    m_width,m_height;
    wxList m_images;

 public:

  wxImageList(void);

  wxImageList(const int width, const int height, const Bool mask = TRUE, const int initial = 1)
    { Create(width, height, mask, initial); };
  ~wxImageList(void);

  int GetImageCount(void);
  Bool Create(const int width, const int height, const Bool mask = TRUE, const int initialNumber = 1);
  int Add( wxBitmap *bitmap, const wxBitmap *mask = NULL);
  int Add( wxBitmap *bitmap, const wxColour& maskColour);
  int Add( const wxIcon *icon );
  Bool Replace( const int index, const wxBitmap *bitmap, const wxBitmap *mask = NULL );
//  Bool Replace( const int index, const wxBitmap *bitmap, const wxColour& maskColour );
  Bool Replace( const int index, const wxIcon *icon );
  Bool Remove( const int index );
  Bool RemoveAll(void);
  Bool GetSize( const int index, int &width, int &height );
  Bool Draw(const int index, wxDC& dc, const int x, const int y,
    const int flags = wxIMAGELIST_DRAW_NORMAL, const Bool solidBackground = FALSE);


    
/*
  wxIcon *MakeIcon(const int index);
  Bool SetOverlayImage(const int index, const int overlayMask);

*/

  // TODO: Drag-and-drop related functionality.

#if 0
  // Creates a new drag image by combining the given image (typically a mouse cursor image)
  // with the current drag image.
  Bool SetDragCursorImage(const int index, const wxPoint& hotSpot);

  // If successful, returns a pointer to the temporary image list that is used for dragging;
  // otherwise, NULL.
  // dragPos: receives the current drag position.
  // hotSpot: receives the offset of the drag image relative to the drag position.
  static wxImageList *GetDragImageList(wxPoint& dragPos, wxPoint& hotSpot);

  // Call this function to begin dragging an image. This function creates a temporary image list
  // that is used for dragging. The image combines the specified image and its mask with the
  // current cursor. In response to subsequent mouse move messages, you can move the drag image
  // by using the DragMove member function. To end the drag operation, you can use the EndDrag
  // member function.
  Bool BeginDrag(const int index, const wxPoint& hotSpot);

  // Ends a drag operation.
  Bool EndDrag(void);

  // Call this function to move the image that is being dragged during a drag-and-drop operation.
  // This function is typically called in response to a mouse move message. To begin a drag
  // operation, use the BeginDrag member function.
  static Bool DragMove(const wxPoint& point);

  // During a drag operation, locks updates to the window specified by lockWindow and displays
  // the drag image at the position specified by point.
  // The coordinates are relative to the window's upper left corner, so you must compensate
  // for the widths of window elements, such as the border, title bar, and menu bar, when
  // specifying the coordinates.
  // If lockWindow is NULL, this function draws the image in the display context associated
  // with the desktop window, and coordinates are relative to the upper left corner of the screen.
  // This function locks all other updates to the given window during the drag operation.
  // If you need to do any drawing during a drag operation, such as highlighting the target
  // of a drag-and-drop operation, you can temporarily hide the dragged image by using the
  // wxImageList::DragLeave function.

  // lockWindow: pointer to the window that owns the drag image.
  // point:      position at which to display the drag image. Coordinates are relative to the
  //             upper left corner of the window (not the client area).

  static Bool DragEnter( wxWindow *lockWindow, const wxPoint& point );

  // Unlocks the window specified by pWndLock and hides the drag image, allowing the
  // window to be updated.
  static Bool DragLeave( wxWindow *lockWindow );

  /* Here's roughly how you'd use these functions if implemented in this Win95-like way:

  1) Starting to drag:

  wxImageList *dragImageList = new wxImageList(16, 16, TRUE);
  dragImageList->Add(myDragImage); // Provide an image to combine with the current cursor
  dragImageList->BeginDrag(0, wxPoint(0, 0));
  wxShowCursor(FALSE);        // wxShowCursor not yet implemented in wxWin
  myWindow->CaptureMouse();

  2) Dragging:

  // Called within mouse move event. Could also use dragImageList instead of assuming
  // these are static functions.
  // These two functions could possibly be combined into one, since DragEnter is
  // a bit obscure.
  wxImageList::DragMove(wxPoint(x, y));  // x, y are current cursor position
  wxImageList::DragEnter(NULL, wxPoint(x, y)); // NULL assumes dragging across whole screen

  3) Finishing dragging:

  dragImageList->EndDrag();
  myWindow->ReleaseMouse();
  wxShowCursor(TRUE);
*/

#endif

  // Implementation
  ////////////////////////////////////////////////////////////////////////////

};

#endif

