/*
 * Copyright: (C) 1997, GNU (Robert Roebling)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef __GNUG__
#pragma implementation "WIN_dirtree.h"
#endif

#define Uses_wxPanel
#define Uses_wxCanvas
#define Uses_wxFrame
#define Uses_wxString
#define Uses_wxDialogBox
#define Uses_wxButton
#define Uses_wxText
#define Uses_wxCheckBox
#define Uses_wxRadioBox
#define Uses_wxMessage
#define Uses_wxTimer
#include "wx.h"

#include "strings.h"

#include "WIN_rawtext.h"
#include "WIN_dirtree.h"
#include "FMDialogs.h"

//-----------------------------------------------------------------------------
//  wxDirBranch
//-----------------------------------------------------------------------------

IMPLEMENT_DYNAMIC_CLASS(wxDirBranch,wxVirtualBranch);

wxDirBranch::wxDirBranch( char *dir, wxFileCtrl *f  ) :
  wxVirtualBranch( FALSE, "dir", 0 )
{
  m_showHidden = FALSE;
  dirName = dir;
  if (dirName == "/")
  {
    bName = "The Computer";
    SetHasChildren( TRUE );
  }
  else 
  if (dirName == "/home")
  {
    bName = "My Home";
    SetHasChildren( TRUE );
    dirName += "/";
    char buf[100];
    wxGetUserId( buf, 100 );
    dirName += buf;
  }
  else
  if (dirName == "/proc")
  {
    bName = "Info Filesystem";
    SetHasChildren( TRUE );
  }
  else 
  if (dirName == "/mnt")
  {
    bName = "Mounted Devices";
    SetHasChildren( TRUE );
  }
  else 
    bName = wxFileNameFromPath( dir ); 
  files = f;
};

char *wxDirBranch::GetName( void )
{
  return (char*)(bName.GetData());
};

char *wxDirBranch::GetDirName( void )
{
  return (char*)(dirName.GetData());
};

void wxDirBranch::Expand( void )
{
  wxString res = dirName + "/*";
  char *f = wxFindFirstFile( res.GetData(), wxDIR );
  if (!f) return;
  do 
  {
    res = wxFileNameFromPath( f );
    if (m_showHidden || (res[0] != '.'))
    {
      if ((res != ".") && (res != "..") &&
          (strcmp( f, "/home") != 0) &&
          (strcmp( f, "/proc") != 0) &&
          (strcmp( f, "/mnt") != 0) )
        AddChild( new wxDirBranch( f, files ) );
    };
    f = wxFindNextFile();
  } while (f);

  wxNode *node = GetChildrenList()->First();
  while (node)
  {
    wxDirBranch *b = (wxDirBranch*)node->Data();
    res = b->GetDirName();
    res += "/*";
    f = wxFindFirstFile( res.GetData(), wxDIR );
    f = wxFindNextFile();
    f = wxFindNextFile();
    if (f) b->SetHasChildren( TRUE );
    node = node->Next();
  };

  SortChildren();
};

void wxDirBranch::Activated( void )
{
  if (files) files->NewDir( dirName );
};

int wxDirBranch::CompareTo( wxVirtualBranch *b )
{
  return strcmp( GetName(), b->GetName() );
};

//-----------------------------------------------------------------------------
//  wxDirTree
//-----------------------------------------------------------------------------

IMPLEMENT_DYNAMIC_CLASS(wxDirTree,wxVirtualTree);
  
wxDirTree::wxDirTree( wxPanel *p, int x, int y, int width, int height,
  long style, Constdata char *name ) :
  wxVirtualTree( p, x, y, width, height, style, name )
{
};

void wxDirTree::OnDropFiles( int n, char **files, int x, int y )
{
  wxDirBranch *db = (wxDirBranch*)GetHit( x, y );
  if (db)
  {
    wxString s = db->GetDirName();
    wxCopyMoveDia( (wxFrame*)GetParent(), n, files, s );
  };
};


