/*
 * Copyright: (C) 1997, GNU (Robert Roebling)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef __GNUG__
#pragma implementation "WIN_filelist.h"
#endif

#define Uses_wxPanel
#define Uses_wxFrame
#define Uses_wxCanvas
#define Uses_wxString
#define Uses_wxDialogBox
#define Uses_wxButton
#define Uses_wxText
#define Uses_wxCheckBox
#define Uses_wxRadioBox
#define Uses_wxMessage
#define Uses_wxTimer
#define Uses_XtIntrinsic
#include "wx.h"

#include "WIN_rawtext.h"
#include "WIN_filelist.h"
#include "FMDialogs.h"

#include "sys/types.h"
#include "sys/stat.h"
#include "dirent.h"
#include "pwd.h"
#include "grp.h"
#include "time.h"

//-----------------------------------------------------------------------------
//  wxFileData
//-----------------------------------------------------------------------------

IMPLEMENT_DYNAMIC_CLASS(wxFileData,wxObject);
                                               
wxFileData::wxFileData( char *n, char *fname ) :
  wxObject()
{
  m_name = n;
  m_fileName = fname;

  struct stat buff;
  stat( m_fileName.GetData(), &buff );
  struct stat lbuff;
  lstat( m_fileName.GetData(), &lbuff );

  struct tm *t = localtime( &lbuff.st_mtime );
//  struct passwd *user = getpwuid( buff.st_uid );
//  struct group *grp = getgrgid( buff.st_gid );

  m_isDir = S_ISDIR( buff.st_mode );
  m_isLink = S_ISLNK( lbuff.st_mode );
  m_isExe = ((buff.st_mode & S_IXUSR ) == S_IXUSR );

  m_size = buff.st_size;

  m_hour = t->tm_hour;
  m_minute = t->tm_min;
  m_month = t->tm_mon+1;
  m_day = t->tm_mday;
  m_year = t->tm_year;

  m_permissions.sprintf( "%c%c%c", 
     ((( buff.st_mode & S_IRUSR ) == S_IRUSR ) ? 'r' : '-'),
     ((( buff.st_mode & S_IWUSR ) == S_IWUSR ) ? 'w' : '-'),
     ((( buff.st_mode & S_IXUSR ) == S_IXUSR ) ? 'x' : '-') );
};

void wxFileData::GetName( wxString &s )
{
  s = m_name;
};   

void wxFileData::GetFullName( wxString &s )
{
  s = m_fileName;
};   

void wxFileData::GetHint( wxString &s )
{
  s = m_fileName;
  s += "  ";
  if (m_isDir) s += "<DIR> ";
  else if (m_isLink) s += "<LINK> ";
  else
  {
    s += wxLongToString( m_size );
    s += " bytes ";
  };
  s += wxIntToString( m_day );
  s += ".";
  s += wxIntToString( m_month );
  s += ".";
  s += wxIntToString( m_year );
  s += "  ";
  s += wxIntToString( m_hour );
  s += ":";
  s += wxIntToString( m_minute );
  s += "  ";
  s += m_permissions;
};   
    
Bool wxFileData::IsDir( void )
{
  return m_isDir;
};

Bool wxFileData::IsExe( void )
{
  return m_isExe;
};

Bool wxFileData::IsLink( void )
{
  return m_isLink;
};

long wxFileData::GetSize( void )
{
  return m_size;
};

void wxFileData::GetEntry( const int num, wxString &s )
{
  switch (num)
  {
    case 0:
      s = m_name;
      break;
    case 1:
      if (m_isDir) s = "<DIR>";
      else if (m_isLink) s = "<LINK>";
      else s = wxLongToString( m_size );
      break;
    case 2:
      if (m_day < 10) s = "0"; else s = "";
      s += wxIntToString( m_day );
      s += ".";
      if (m_month < 10) s += "0"; 
      s += wxIntToString( m_month );
      s += ".";
      if (m_year < 10) s += "0"; // this should happen real soon...
      s += wxIntToString( m_year );
      break;
    case 3:
      if (m_hour < 10) s = "0"; else s = "";
      s += wxIntToString( m_hour );
      s += ":";
      if (m_minute < 10) s += "0";
      s += wxIntToString( m_minute );
      break;
    case 4:
      s = m_permissions;
      break;
    default:
      s = "No entry";
      break;
  };
};  

Bool wxFileData::NewNameIsLegal( const wxString &WXUNUSED(s) )
{
  return FALSE;
};

void wxFileData::Rename( const wxString &WXUNUSED(s) )
{
};
  
//-----------------------------------------------------------------------------
//  wxFileCtrl
//-----------------------------------------------------------------------------

IMPLEMENT_DYNAMIC_CLASS(wxFileCtrl,wxListCtrl);

wxFileCtrl::wxFileCtrl( wxPanel *p, int x, int y, int width, int height, char* dirName, 
   wxFrame *statusLineFrame, long style, Constdata char *name ) :
  wxListCtrl( p, x, y, width, height, style, name )
{
  m_dirName = dirName;
  m_statusLineFrame = statusLineFrame;
  m_showHidden = FALSE;
  Update();
};

void wxFileCtrl::GetDir( wxString &s )
{
  s = m_dirName;
};

void wxFileCtrl::NewDir( char *dirName )
{
  m_dirName = dirName;
  Update();
  RealizeChanges();
};

void wxFileCtrl::ChangeToListMode()
{
  SetListStyleFlag( wxLC_LIST );
  Update();
  RealizeChanges();
};

void wxFileCtrl::ChangeToReportMode()
{
  SetListStyleFlag( wxLC_REPORT );
  Update();
  RealizeChanges();
};

void wxFileCtrl::ChangeToIconMode()
{
  SetListStyleFlag( wxLC_ICON );
  Update();
  RealizeChanges();
};

void wxFileCtrl::ShowHidden( Bool show )
{
  m_showHidden = show;
  Update();
  RealizeChanges();
};

int ListCompare( const long data1, const long data2, const long WXUNUSED(data) )
{
   wxFileData *fd1 = (wxFileData*)data1 ;
   wxFileData *fd2 = (wxFileData*)data2 ;
   if (fd1->IsDir() && !fd2->IsDir()) return -1;
   if (fd2->IsDir() && !fd1->IsDir()) return 1;
   wxString s1,s2;
   fd1->GetName( s1 );
   fd2->GetName( s2 );
   return strcmp( s1.GetData(), s2.GetData() );
};

void wxFileCtrl::Update( void )
{ 
  DeleteAllItems();
  long my_style = RawGetMode();
  if (my_style & wxLC_REPORT)
  {
    InsertColumn( 0, "Name", wxLIST_FORMAT_LEFT, 110 );
    InsertColumn( 1, "Size", wxLIST_FORMAT_LEFT, 60 );
    InsertColumn( 2, "Date", wxLIST_FORMAT_LEFT, 55 );
    InsertColumn( 3, "Time", wxLIST_FORMAT_LEFT, 50 );
    InsertColumn( 4, "Permissions", wxLIST_FORMAT_LEFT, 120 );
  };
  wxFileData *fd = NULL;
  wxListItem item;
  item.m_mask = wxLIST_MASK_TEXT + wxLIST_MASK_DATA;
  if (my_style & wxLC_ICON) item.m_mask += wxLIST_MASK_IMAGE;
  item.m_itemId = 0;
  item.m_col = 0;
  wxString s;
  wxString res = m_dirName + "/*";
  wxColour *dg = wxTheColourDatabase->FindColour( "MEDIUM GREY" );
  char *f = wxFindFirstFile( res.GetData() );
  while (f)   
  {
    res = wxFileNameFromPath( f );
    fd = new wxFileData( res, f );
    fd->GetName( s );
    if (m_showHidden || (s[0] != '.'))
    {
      fd->GetName( item.m_text );
      item.m_colour = wxBLACK;
      if (fd->IsExe()) item.m_colour = wxRED;
      if (fd->IsDir()) item.m_colour = wxBLUE;
      if (fd->IsLink()) item.m_colour = dg;
      item.m_data = (long)fd;
      if (my_style & wxLC_REPORT)
      {
        InsertItem( item );
        for (int i = 1; i < 5; i++) { fd->GetEntry( i, s ); SetItem( item.m_itemId, i, s ); };
      }
      else if (my_style & wxLC_LIST)
      {
        InsertItem( item );
      } 
     else if (my_style & wxLC_ICON)
      {
        if (fd->IsDir()) item.m_image = 0; else item.m_image = 1;
        InsertItem( item );
      };
      item.m_itemId++;
    };
    f = wxFindNextFile();
  };
  SortItems( ListCompare, 0 );
  s = m_dirName;
  int pos = s.Last( '/' );  
  if ((pos >= 0) && (s != "/"))
  {
    s.Remove( pos, s.Length()-pos );
    if (s.Length() == 0) s = "/";
    fd = new wxFileData( "..", s );
    item.m_itemId = 0;
    item.m_image = 0;
    item.m_text = "..    ";
    item.m_colour = wxBLUE;
    item.m_data = (long)fd;
    InsertItem( item );
    if (style & wxLC_REPORT)
      for (int i = 1; i < 5; i++) { fd->GetEntry( i, s ); SetItem( 0, i, s ); };
  };
  RealizeChanges();
};

void wxFileCtrl::OnActivate( const wxListItem &item )
{
  wxFileData *fd = (wxFileData*)item.m_data;
  if (fd->IsDir())
  {
    fd->GetFullName( m_dirName );
    Update();
    Refresh();
    return;
  };
  if (fd->IsExe())
  {
    wxString s;
    fd->GetFullName( s );
    wxExecute( s );
    return;
  };
};

void wxFileCtrl::OnDropFiles( int n, char **data, int x, int y )
{
  wxString destDir;
  wxPoint pt( x, y );
  int flag = wxLIST_HITTEST_ONITEM;
  long hit = HitTest( pt, flag );
  if (hit > -1)
  {
    wxListItem li;
    li.m_itemId = hit;
    GetItem( li );
    wxFileData *fd = (wxFileData*)li.m_data;
    if (fd->IsDir()) fd->GetFullName( destDir );
  };
  if (destDir.IsNull()) destDir = m_dirName;
  wxCopyMoveDia( (wxFrame*)GetParent(), n, data, destDir );
};

Bool wxFileCtrl::OnListNotify( wxListEvent& event )
{
  switch ( event.GetEventType() )
  {
    case wxEVT_COMMAND_LIST_KEY_DOWN:
    {
      OnActivate( event.m_item );
      return TRUE;
    };
    case wxEVT_COMMAND_LIST_ITEM_FOCUSSED:
    {
      if (m_statusLineFrame)
      {
        wxString s;
	wxFileData *fd = (wxFileData*)event.m_item.m_data;
	fd->GetHint( s );
	m_statusLineFrame->SetStatusText( s );
      };
      return TRUE;
    };
    case wxEVT_COMMAND_LIST_DELETE_ITEM:
    {
      wxFileData *fd = (wxFileData*)event.m_item.m_data;
      delete fd;
      return TRUE;
    };
    case wxEVT_COMMAND_LIST_END_LABEL_EDIT:
    {
      wxFileData *fd = (wxFileData*)event.m_item.m_data;
      wxString newName = event.m_item.m_text;
      if (fd->NewNameIsLegal( newName ))
      {
        // rename
	fd->Rename( newName );
	SetItem( event.m_item );
	RealizeChanges();
      }
      else
      {
        wxMessageBox( "Name exists already or is invalid.", "Rename error." );
      };
      return TRUE;
    };
    default:
    {
      return FALSE;
    };
  };
  return FALSE;
};

void wxFileCtrl::StartDragging( const int WXUNUSED(x), const int WXUNUSED(y) )  // gets called by wxDragCanvas
{
  long ret = -1;
  wxString s;
  for (;;)
  {
    ret = GetNextItem( ret, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED );
    if (ret == -1) return;
    wxListItem item;
    item.m_itemId = ret;
    GetItem( item );
    wxFileData *fd = (wxFileData*)item.m_data;
    fd->GetFullName( s );
    AddDragString( s );
    ret++;
  };
};


