/*
 * Program: live (Linux visual Environment)
 * 
 * Purpose: visual system administration
 *
 * Author: Robert Roebling
 *
 * Copyright: (C) 1997, GNU (Robert Roebling)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef FileList_h
#define FileList_h

#ifdef __GNUG__
#pragma interface
#endif

#include "WIN_listctrl.h"

//-----------------------------------------------------------------------------
// derived classes
//-----------------------------------------------------------------------------

class wxFileData;
class wxFileCtrl;

//-----------------------------------------------------------------------------
//  wxFileData
//-----------------------------------------------------------------------------

class wxFileData : public wxObject
{
  DECLARE_DYNAMIC_CLASS(wxFileData);

  private:
    wxString m_name;
    wxString m_fileName;
    long     m_size;
    int      m_hour;
    int      m_minute;
    int      m_year;
    int      m_month;
    int      m_day;
    wxString m_permissions;
    Bool     m_isDir;
    Bool     m_isLink;
    Bool     m_isExe;

  public:
    wxFileData( void ) {};
    wxFileData( char *n, char *fname );
    void GetName( wxString &s );
    void GetFullName( wxString &s );
    void GetHint( wxString &s );
    void GetEntry( wxString &s, int num );
    Bool IsDir( void );
    Bool IsLink( void );
    Bool IsExe( void );
    long GetSize( void );
    void GetEntry( const int num, wxString &s );
    Bool NewNameIsLegal( const wxString &s );
    void Rename( const wxString &s );
};

//-----------------------------------------------------------------------------
//  wxFileCtrl
//-----------------------------------------------------------------------------

class wxFileCtrl : public wxListCtrl
{
  DECLARE_DYNAMIC_CLASS(wxFileCtrl);
    
  private:
    wxString      m_dirName;
    wxFrame      *m_statusLineFrame;
    Bool          m_showHidden;

  public:
    wxFileCtrl( void ) 
      : wxListCtrl() {}; 
    wxFileCtrl( wxPanel *p, int x, int y, int width, int height, char *dirName,
      wxFrame *statusLineFrame = NULL, long style=wxLC_LIST, Constdata char *name= "wxFileCtrl" );
    void NewDir( char *dirName );
    void GetDir( wxString &s );
    void ChangeToListMode();
    void ChangeToReportMode();
    void ChangeToIconMode();
    void ShowHidden( Bool show = TRUE );
    void Update( void );
    void OnActivate( const wxListItem &item );
    virtual void OnDropFiles( int n, char **files, int x, int y );
    virtual Bool OnListNotify( wxListEvent &event );
    virtual void StartDragging( const int x, const int y );
};

#endif // FileList_h
