/*
 * File:	WIN_listctrl.cc
 * Purpose:	wxXt list control implementation
 * Author:	Robert Roebling
 * Created:	1997
 * Copyright:   L-GPL (Robert Roebling)
 */

#ifdef __GNUG__
#pragma implementation "WIN_listctrl.h"
#endif

#define Uses_wxPanel
#define Uses_wxCanvas
#define Uses_wxString
#define Uses_wxText
#include "wx.h"

#include "WIN_listctrl.h"

// -------------------------------------------------------------------------------------
// wxListItem
// -------------------------------------------------------------------------------------

IMPLEMENT_DYNAMIC_CLASS(wxListItem, wxObject)

wxListItem::wxListItem(void)
{
  m_mask = 0;
  m_itemId = 0;
  m_col = 0;
  m_state = 0;
  m_stateMask = 0;
  m_image = 0;
  m_data = 0;
  m_format = wxLIST_FORMAT_CENTRE;
  m_width = 0;
  m_colour = wxBLACK;
}

// -------------------------------------------------------------------------------------
// wxListEvent
// -------------------------------------------------------------------------------------

IMPLEMENT_DYNAMIC_CLASS(wxListEvent, wxCommandEvent)

wxListEvent::wxListEvent(WXTYPE commandType, int WXUNUSED(id)):
  wxCommandEvent(commandType)
{
// eventId = id;
  m_code = 0;
  m_itemIndex = 0;
  m_col = 0;
  m_cancelled = FALSE;
};

// -------------------------------------------------------------------------------------
// wxListCtrl
// -------------------------------------------------------------------------------------

IMPLEMENT_DYNAMIC_CLASS(wxListCtrl, wxRawListCtrl)

wxListCtrl::wxListCtrl(void) : 
  wxRawListCtrl( )
{
  m_imageListNormal = NULL;
  m_imageListSmall = NULL;
  m_imageListState = NULL;
  m_style = wxLC_LIST;
}

wxListCtrl::wxListCtrl( wxWindow *parent, int x, int y, int w, int h, const long style, const char *name ) :
  wxRawListCtrl( parent, x, y, w, h, style, style, name )

{
  m_imageListNormal = NULL;
  m_imageListSmall = NULL;
  m_imageListState = NULL;
  m_style = style;
}

wxListCtrl::~wxListCtrl(void)
{
}

void wxListCtrl::SetSingleStyle( const long style, const Bool add )
{
  // This code in from Julian. Looks cool. Hope it works.
  long flag = RawGetMode();
  // Get rid of conflicting styles
  if (add)
  {
    if (style & wxLC_MASK_TYPE)	flag = flag & ~wxLC_MASK_TYPE;
    if (style & wxLC_MASK_ALIGN) flag = flag & ~wxLC_MASK_ALIGN;
    if (style & wxLC_MASK_SORT) flag = flag & ~wxLC_MASK_SORT;
  };
  if (flag & style)
  {
    if (!add) flag -= style;
  }
  else
  {
    if (add)
    {
      flag |= style;
    };
  };
  m_style = flag;
  RawSetMode( m_style );
};

void wxListCtrl::SetListStyleFlag( const long flag )
{
  m_style = flag;
  RawSetMode( flag );
};

void wxListCtrl::RealizeChanges( void )
{ 
  RawRealizeChanges();
};

void wxListCtrl::SetBackgroundColour(const wxColour& col)
{
  // This is from Julian. You know.
  // Not in wxWin 1.xx ???
  wxWindow::SetBackgroundColour( (wxColour&)col );
};

Bool wxListCtrl::GetColumn(const int col, wxListItem &item) 
{
  RawGetColumn( col, item );
  return TRUE;
};

Bool wxListCtrl::SetColumn( const int col, wxListItem& item )
{
  RawSetColumn( col, item );
  return TRUE;
};

int wxListCtrl::GetColumnWidth( const int col ) 
{
  return RawGetColumnWidth( col );
};

Bool wxListCtrl::SetColumnWidth( const int col, const int width )
{
  RawSetColumnWidth( col, width );
  return TRUE;
};

int wxListCtrl::GetCountPerPage(void) 
{
  return RawGetCountPerPage();  // different from Windows ?
};

/*
wxText& wxListCtrl::GetEditControl(void) const
{
};
*/

Bool wxListCtrl::GetItem( wxListItem &info ) 
{
  RawGetItem( info );
  return TRUE;
};

Bool wxListCtrl::SetItem( wxListItem &info )
{
  RawSetItem( info );
  return TRUE;
};

long wxListCtrl::SetItem( const long index, const int col, const wxString& label, const int imageId )
{
  wxListItem info;
  info.m_text = label;
  info.m_mask = wxLIST_MASK_TEXT;
  info.m_itemId = index;
  info.m_col = col;
  if ( imageId > -1 )
  {
    info.m_image = imageId;
    info.m_mask |= wxLIST_MASK_IMAGE;
  }
;
  RawSetItem(info);
  return TRUE;
};

int wxListCtrl::GetItemState( const long item, const long stateMask )
{
  return RawGetItemState( item, stateMask );  
};

Bool wxListCtrl::SetItemState( const long item, const long state, const long stateMask )
{
  RawSetItemState( item, state, stateMask );
  return TRUE;
};

Bool wxListCtrl::SetItemImage( const long item, const int image, const int WXUNUSED(selImage) )
{
  wxListItem info;
  info.m_image = image;
  info.m_mask = wxLIST_MASK_IMAGE;
  info.m_itemId = item;
  RawSetItem( info );
  return TRUE;
};

wxString wxListCtrl::GetItemText( const long item ) 
{
  wxListItem info;
  info.m_itemId = item;
  RawGetItem( info );
  return info.m_text;
};

void wxListCtrl::SetItemText( const long item, const wxString &str )
{
  wxListItem info;
  info.m_mask = wxLIST_MASK_TEXT;
  info.m_itemId = item;
  info.m_text = str;
  RawSetItem( info );
};

long wxListCtrl::GetItemData( const long item )
{
  wxListItem info;
  info.m_itemId = item;
  RawGetItem( info );
  return info.m_data;
};

Bool wxListCtrl::SetItemData( const long item, long data )
{
  wxListItem info;
  info.m_mask = wxLIST_MASK_DATA;
  info.m_itemId = item;
  info.m_data = data;
  RawSetItem( info );
  return TRUE;
};

Bool wxListCtrl::GetItemRect( const long item, wxRectangle &rect,  const int WXUNUSED(code) )
{
  RawGetItemRect( item, rect );
  return TRUE;
};

Bool wxListCtrl::GetItemPosition( const long WXUNUSED(item), wxPoint& WXUNUSED(pos) ) const
{
  return 0;
};

Bool wxListCtrl::SetItemPosition( const long WXUNUSED(item), const wxPoint& WXUNUSED(pos) )
{
  return 0;
};

int wxListCtrl::GetItemCount(void) 
{
  return RawGetItemCount();
};

int wxListCtrl::GetItemSpacing( Bool isSmall )
{
  return RawGetItemSpacing( isSmall );
};

int wxListCtrl::GetSelectedItemCount(void) 
{
  return RawGetSelectedItemCount();
};

/*
wxColour wxListCtrl::GetTextColour(void) const
{
};

void wxListCtrl::SetTextColour(const wxColour& WXUNUSED(col))
{
};
*/

long wxListCtrl::GetTopItem(void)
{
  return 0;
};

long wxListCtrl::GetNextItem( const long item, int geom, int state )
{
  return RawGetNextItem( item, geom, state );
};

wxImageList *wxListCtrl::GetImageList(const int which)
{
  if (which == wxIMAGE_LIST_NORMAL)
  {
    return m_imageListNormal;
  }
  else if (which == wxIMAGE_LIST_SMALL)
  {
    return m_imageListSmall;
  }
  else if (which == wxIMAGE_LIST_STATE)
  {
    return m_imageListState;
  };
  return NULL;
};

void wxListCtrl::SetImageList( wxImageList *imageList, const int which )
{
  RawSetImageList( imageList, which );
};

Bool wxListCtrl::Arrange( const int WXUNUSED(flag) )
{
  return 0;
};

Bool wxListCtrl::DeleteItem( const long item )
{
  RawDeleteItem( item );
  return TRUE;
};

Bool wxListCtrl::DeleteAllItems(void)
{
  RawDeleteEverything();
  return TRUE;
};

Bool wxListCtrl::DeleteColumn( const int col )
{
  RawDeleteColumn( col );
  return TRUE;
};

/*
wxText& wxListCtrl::Edit( const long WXUNUSED(item ) )
{
};
*/

Bool wxListCtrl::EnsureVisible( const long item )
{
  RawEnsureVisible( item );
  return TRUE;
};

long wxListCtrl::FindItem( const long WXUNUSED(start), const wxString& WXUNUSED(str),
                           const Bool WXUNUSED(partial) )
{
  return 0;
};

long wxListCtrl::FindItem( const long WXUNUSED(start), const long WXUNUSED(data) )
{
  return 0;
};

long wxListCtrl::FindItem( const long WXUNUSED(start), const wxPoint& WXUNUSED(pt), 
                           const int WXUNUSED(direction)) 
{
  return 0;
};

long wxListCtrl::HitTest( const wxPoint &point, int &flags )
{
  return RawHitTest( (int)point.x, (int)point.y, flags );
};

long wxListCtrl::InsertItem( wxListItem& info )
{
  RawInsertItem( info );
  return 0;
};

long wxListCtrl::InsertItem( const long index, const wxString &label )
{
  wxListItem info;
  info.m_text = label;
  info.m_mask = wxLIST_MASK_TEXT;
  info.m_itemId = index;
  return InsertItem( info );
};

long wxListCtrl::InsertItem( const long index, const int imageIndex )
{
  wxListItem info;
  info.m_mask = wxLIST_MASK_IMAGE;
  info.m_image = imageIndex;
  info.m_itemId = index;
  return InsertItem( info );
};

long wxListCtrl::InsertItem( const long index, const wxString &label, const int imageIndex )
{
  wxListItem info;
  info.m_text = label;
  info.m_image = imageIndex;
  info.m_mask = wxLIST_MASK_TEXT | wxLIST_MASK_IMAGE;
  info.m_itemId = index;
  return InsertItem( info );
};

long wxListCtrl::InsertColumn( const long col, wxListItem &item )
{
  RawInsertColumn( col, item );
  return 0;
};

long wxListCtrl::InsertColumn( const long col, const wxString &heading,
                               const int format, const int width )
{
  wxListItem item;
  item.m_mask = wxLIST_MASK_TEXT | wxLIST_MASK_FORMAT;
  item.m_text = heading;
  if (width >= -2)
  {
    item.m_mask |= wxLIST_MASK_WIDTH;
    item.m_width = width;
  }
;
  item.m_format = format;

  return InsertColumn( col, item );
};

Bool wxListCtrl::ScrollList( const int WXUNUSED(dx), const int WXUNUSED(dy) )
{
  return 0;
};

// Sort items.
// fn is a function which takes 3 long arguments: item1, item2, data.
// item1 is the long data associated with a first item (NOT the index).
// item2 is the long data associated with a second item (NOT the index).
// data is the same value as passed to SortItems.
// The return value is a negative number if the first item should precede the second
// item, a positive number of the second item should precede the first,
// or zero if the two items are equivalent.
// data is arbitrary data to be passed to the sort function.

Bool wxListCtrl::SortItems( wxListCtrlCompare fn, long data )
{
  RawSortItems( fn, data );
  return TRUE;
};


