/*
 * Copyright: (C) 1997, GNU (Robert Roebling)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef __GNUG__
#pragma implementation "WIN_rawtext.h"
#endif

#define Uses_wxPanel
#define Uses_wxCanvas
#define Uses_wxString
#define Uses_wxTimer
#include "wx.h"

#include "WIN_rawtext.h"

//-----------------------------------------------------------------------------
//  wxCaret
//-----------------------------------------------------------------------------

IMPLEMENT_DYNAMIC_CLASS(wxCaret,wxTimer);

wxCaret::wxCaret(void) :
  wxTimer()
{
  m_visible = FALSE;
  m_blinked_out = TRUE;
  m_font = NULL;
  m_owner = NULL;
};

wxCaret::~wxCaret(void)
{
  Stop();
//  if (m_visible && !m_blinked_out) DoDraw();
};

void wxCaret::SetOwnerAndFont( wxWindow *owner, wxFont *font )
{
  m_owner = owner;
  m_font = font;
};

void wxCaret::DoDraw(void)
{
  wxCanvasDC *dc = m_owner->GetDC();
  wxPen *oldPen = dc->GetPen();
  dc->BeginDrawing();
  int oldLF = dc->GetLogicalFunction();
  dc->SetLogicalFunction( wxXOR );
  dc->SetPen( wxBLACK_PEN );
  dc->DrawLine( m_x, m_y, m_x+m_w, m_y );
  dc->DrawLine( m_x, m_y+m_h, m_x+m_w, m_y+m_h );
  dc->DrawLine( m_x+(m_w/2), m_y, m_x+(m_w/2), m_y+m_h );
  dc->SetLogicalFunction( oldLF );
  dc->SetPen( oldPen );
  dc->EndDrawing();
};

void wxCaret::Show( int x, int y, Bool blinking )
{
  if (!m_owner) return;
  if (!m_font) return;
  Stop();
  if (m_visible && !m_blinked_out) DoDraw();
  m_x = x;
  m_y = y;
  wxCanvasDC *dc = m_owner->GetDC();
  wxFont *oldFont = dc->GetFont();
  float fw,fh;
  dc->SetFont( m_font );
  dc->GetTextExtent( "H", &fw, &fh );
  dc->SetFont( oldFont );
  m_w = (int)(fw*0.66);
  m_h = (int)fh-1;
  if (m_w % 2) m_w--;
  m_x -= m_w/2;
  m_blinked_out = FALSE;
  m_visible = TRUE;
  DoDraw();
  Start( 600, !blinking );
};

void wxCaret::Hide()
{
  if (!m_owner) return;
  if (!m_font) return;
  Stop();
  if (!m_blinked_out) DoDraw();
  m_visible = FALSE;
};

void wxCaret::Notify()
{
  DoDraw();
  m_blinked_out = !m_blinked_out;
};

Bool wxCaret::IsVisible(void)
{
  return m_visible;
};

//-----------------------------------------------------------------------------
//  wxRawText
//-----------------------------------------------------------------------------

IMPLEMENT_DYNAMIC_CLASS(wxRawText,wxCanvas);

wxRawText::wxRawText( wxWindow *parent, const char *value, 
      int x, int y, int w, int h, int style, char *name ) : 
  wxCanvas( parent, x, y, w, h, style, name )
{  
  m_text = value;
  m_cursorPos = 0;
  m_startSel = 0;
  m_endSel = 0;
  m_scrollPos = 0;
  m_style = style;
  m_editable = TRUE;
  wxColour *hl = new wxColour( 208, 180, 176 );  // deallocate ?
  m_hilightBrush = new wxBrush( (*hl), wxSOLID );
  m_myFont = ((wxPanel*)GetParent())->GetButtonFont();
  m_caret.SetOwnerAndFont( this, m_myFont );
};

int wxRawText::GetStringIndex( int x )
{
  if (m_text.Length() == 0) return 0;
  wxString test;
  wxCanvasDC *dc = GetDC();
  dc->SetFont( m_myFont );
  for (int i = 0; i < (int)m_text.Length(); i++)
  {
    test.Append( m_text[i] );
    float fh = 0;
    float fw = 0;
    dc->GetTextExtent( test, &fw, &fh );
    if (fw > x) return i;
  };
  return m_text.Length();
};

int wxRawText::GetStringLength( int pos )
{
  if (pos <= 0) return 3;
  wxString test = m_text;
  test.Remove( pos );
  wxCanvasDC *dc = GetDC();
  float fh = 0;
  float fw = 0;
  dc->SetFont( m_myFont );
  dc->GetTextExtent( test, &fw, &fh );
  return 5+(int)fw;
};

void wxRawText::UpdateCursor(void)
{
  m_caret.Show( GetStringLength(m_cursorPos), 2 );
};

void wxRawText::RefreshLeft(void)
{
  int w = 0;
  int h = 0;
  GetClientSize( &w, &h );
  int start = GetStringLength(m_cursorPos-1);
  wxRectangle rect( start, 0, w-start, h );
  Refresh( TRUE, &rect );
};

void wxRawText::OnChar( wxKeyEvent &event )
{
  switch (event.keyCode) 
  {
    case WXK_DELETE:
    {
      if (m_startSel-m_endSel == 0)
      {
        if (m_cursorPos != (int)m_text.Length())
        {
          m_text.Remove( m_cursorPos, 1 );
	  m_caret.Hide();
	  RefreshLeft();
	  UpdateCursor();
        };
      }
      else
        Delete(); 
      return;
    };
    case WXK_BACK:
    {
      if (m_startSel-m_endSel == 0)
      {
        if (m_cursorPos != 0)
        {
	  m_cursorPos--;
          m_text.Remove( m_cursorPos, 1 );
	  m_caret.Hide();
	  RefreshLeft();
	  UpdateCursor();
        };
      }
      else
        Delete(); 
      return;;
    };
    case WXK_TAB:
    {
      if (event.ShiftDown()) 
	TravPrev( &event );
      else                   
	TravNext( &event );
      return;
    };
    case WXK_UP: TravUp( &event ); return;
    case WXK_DOWN: TravDown( &event ); return;
    case WXK_HOME: m_cursorPos = 0; UpdateCursor(); return;
    case WXK_END: m_cursorPos = m_text.Length(); UpdateCursor(); return;
    case WXK_LEFT:
    {
      if (m_cursorPos != 0)
      {
        m_cursorPos--;
	UpdateCursor();
      };
      return;
    };
    case WXK_RIGHT:
    {
      if (m_cursorPos != (int)m_text.Length())
      {
        m_cursorPos++;
	UpdateCursor();
      };
      return;
    };
  };
  if ((event.keyCode >= 32) && (event.keyCode < 300))
  {
    m_text.Insert( m_cursorPos, event.keyCode );
    m_caret.Hide();
    m_cursorPos++;
    RefreshLeft();
    UpdateCursor();
  };
};

void wxRawText::OnEvent( wxMouseEvent &event )
{  
  float x = 0;
  float y = 0;
  event.Position( &x, &y );
  if (event.LeftDown())
  {
    m_cursorPos = GetStringIndex( (int)x-5 );
    UpdateCursor();
   };
};

void wxRawText::OnPaint( void )
{
  wxCanvasDC *dc = GetDC();
  wxPen *oldPen = dc->GetPen();
  wxBrush *oldBrush = dc->GetBrush();
  wxFont* oldFont = dc->GetFont();
  dc->BeginDrawing();

  dc->SetFont( m_myFont );  
  dc->DrawText( m_text, 5, 2 );
    
  dc->EndDrawing();
  dc->SetPen( oldPen );
  dc->SetBrush( oldBrush );
  dc->SetFont( oldFont );
//  UpdateCursor();
};

void wxRawText::OnSetFocus(void)
{
  UpdateCursor();
};

void wxRawText::OnKillFocus(void)
{
  m_caret.Hide();
};

char* wxRawText::GetValue(void)
{  
  return m_text;
};

void wxRawText::SetValue( char *value )
{  
  m_text = value;
  m_cursorPos = 0;
  m_startSel = 0;
  m_endSel = 0;
  m_scrollPos = 0;
  Refresh();
};

int wxRawText::GetInsertPoint(void)
{  
  return m_cursorPos;
};

void wxRawText::SetInsertionPoint( int pos )
{
  m_cursorPos = pos;
  m_startSel = 0;
  m_endSel = 0;
  Refresh();
};

void wxRawText::SetEditbale( Bool editable )
{  
  m_editable = editable;
};

void wxRawText::SetSelection( int from, int to )
{  
  if (from > to) { int dummy = from; from = to; to = dummy; };
  m_startSel = from;
  m_endSel = to;
  Refresh();
};

int wxRawText::GetLastPosition(void)
{
  return m_text.Length();
};

void wxRawText::Remove( int from, int to )
{  
  if (from > to) { int dummy = from; from = to; to = dummy; };
};

void wxRawText::Replace( int from, int to, char *WXUNUSED(value) )
{  
  if (from > to) { int dummy = from; from = to; to = dummy; };
};

void wxRawText::Cut(void)
{  
};

void wxRawText::Copy(void)
{  
};

void wxRawText::Paste(void)
{  
};

void wxRawText::Delete(void)
{  
};
