/*
 *
 * Copyright: (C) 1997, GNU (Robert Roebling)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef RawText_h
#define RawText_h

#ifdef __GNUG__
#pragma interface
#endif

//-----------------------------------------------------------------------------
// classes
//-----------------------------------------------------------------------------

class wxCaret;
class wxRawText;

//-----------------------------------------------------------------------------
//  constants
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//  wxCaret
//-----------------------------------------------------------------------------

class wxCaret : public wxTimer
{
  DECLARE_DYNAMIC_CLASS(wxCaret);

  private:
  
    int          m_x,m_y,m_w,m_h;
    wxWindow    *m_owner;
    Bool         m_visible;
    Bool         m_blinked_out;
    wxFont      *m_font;
    
    void DoDraw(void);
    void Notify(void);
    
  public:
  
    wxCaret(void);
    ~wxCaret(void);
    void SetOwnerAndFont( wxWindow *owner, wxFont *font );
    void Show( int x, int y, Bool blinking = TRUE );
    void Hide(void);
    Bool IsVisible(void);
};

//-----------------------------------------------------------------------------
//  wxRawText
//-----------------------------------------------------------------------------

class wxRawText : public wxCanvas
{
  DECLARE_DYNAMIC_CLASS(wxRawText);

  private:
  
    wxString     m_text;
    int          m_cursorPos;
    int          m_startSel,m_endSel;
    int          m_scrollPos;
    int          m_style;
    Bool         m_editable;
    wxCaret      m_caret;
    wxFont      *m_myFont;
    wxBrush     *m_hilightBrush;
    
  public:
  
    wxRawText(void) : wxCanvas() {};
    wxRawText( wxWindow *parent, const char *value = "", 
      int x = -1, int y = -1, int w = -1, int h = -1, int style = 0, char *name = "rawtext" );
    int GetStringIndex( int x );
    int GetStringLength( int pos );
    void UpdateCursor(void);
    void RefreshLeft(void);
    void OnChar( wxKeyEvent &event );
    void OnEvent( wxMouseEvent &event );
    void OnPaint(void);
    void OnSetFocus(void);
    void OnKillFocus(void);
    char* GetValue(void);
    void SetValue( char *value );
    int GetInsertPoint(void);
    void SetInsertionPoint( int pos );
    void SetEditbale( Bool editable = TRUE );
    void SetSelection( int from, int to );
    int GetLastPosition(void);
    void Remove( int from, int to );
    void Replace( int from, int to, char *value );
    void Cut(void);
    void Copy(void);
    void Paste(void);
    void Delete(void);
  
};

#endif // RawText_h
