/*
 * File:       WIN_toolbar3.cpp
 * Purpose:    Toolbar
 * Author:     Robert Roebling
 * Copyright:  (c) Robert Roebling 1997
 */

#ifdef __GNUG__
#pragma implementation "WIN_toolbar3.h"
#endif

#define Uses_XtIntrinsic
#define Uses_wxMemoryDC
#define Uses_wxScreenDC
#define Uses_wxCanvas
#define Uses_wxFrame
#define Uses_wxString
#define Uses_wxTimer
#define Uses_wxMenuBar
#include "wx.h"

#include "WIN_toolbar3.h"

const wxTOOLBARHEIGHT = 46;

// -----------------------------------------
// wxToolBar3Timer
// -----------------------------------------

IMPLEMENT_DYNAMIC_CLASS(wxToolBar3Timer,wxTimer);

wxToolBar3Timer::wxToolBar3Timer( wxToolBar3 *o ) :
 wxTimer()
{
  owner = o;
};

void wxToolBar3Timer::Notify(void)
{
  owner->DoNotifyHint();
};

// -----------------------------------------
// wxToolBar3Tool
// -----------------------------------------

IMPLEMENT_DYNAMIC_CLASS(wxToolBar3Tool,wxObject);

wxToolBar3Tool::wxToolBar3Tool( wxBitmap *b, int x, int y, int c, Bool s, char *text )
{
  xpos = x;
  ypos = y;
  command = c;
  bitmap = b;
  w = bitmap->GetWidth();
  h = bitmap->GetHeight();
  isEnabled = TRUE;
  isState = s;
  isOn = FALSE;
  isUp = TRUE;
  hint = text;
};

wxToolBar3Tool::~wxToolBar3Tool(void)
{
  delete bitmap;
};

Bool wxToolBar3Tool::IsIn( int x, int y )
{
  return ((xpos-2 <= x) && (xpos+w+2 >= x) &&
			 (ypos-2 <= y) && (ypos+h+2 >= y));
};

void wxToolBar3Tool::Draw( wxToolBar3 *toolBar, Bool up )
{
  if (up == isUp) return;
  isUp = up;
  wxRectangle rect( xpos-2, ypos-2, w+6, h+6 );
  toolBar->Refresh( TRUE, &rect );
};

// -----------------------------------------
// wxToolBar3
// -----------------------------------------

IMPLEMENT_DYNAMIC_CLASS(wxToolBar3,wxPanel);

wxToolBar3::wxToolBar3( wxWindow *parent, wxDecoratedFrame* o, wxToolBar3Pos p ) :
  wxPanel( parent, 0, 0, 200, wxTOOLBARHEIGHT, wxRETAINED | wxNO_KEYB_TRAVERSAL, "wxToolBar3" )
{
  position = p;
  owner = o;
  timer = new wxToolBar3Timer( this );
  state = wxWaiting;
  currentTool = NULL;
  currentHint = NULL;
  timerRunning = FALSE;
//  AllowDoubleClick( TRUE );
};

wxToolBar3::~wxToolBar3()
{
  delete timer;
};

void wxToolBar3::AppendTool( wxBitmap *b, int x, int y, int c, Bool s, char *h )
{
  tools.Append( (wxObject*) new wxToolBar3Tool( b, x, y, c, s, h ) );
  SetSizeAndOwnersSize();
};

wxToolBar3Tool *wxToolBar3::GetToolByCommand( int c )
{
  wxNode *node = tools.First();
  while (node)
  {
    wxToolBar3Tool *t = (wxToolBar3Tool*) node->Data();
    if (t->command == c) return t;
    node = node->Next();
  };
  return NULL;
};

wxToolBar3Tool *wxToolBar3::GetToolByPosition( int x, int y )
{
  wxNode *node = tools.First();
  while (node)
  {
	 wxToolBar3Tool *t = (wxToolBar3Tool*) node->Data();
	 if (t->IsIn( x, y )) return t;
	 node = node->Next();
  };
  return NULL;
};

void wxToolBar3::Enable( int id, Bool flag )
{
  wxNode *node = tools.First();
  while (node)
  {
	 wxToolBar3Tool *t = (wxToolBar3Tool*) node->Data();
	 if (t->command == id) t->isEnabled = flag;
	 node = node->Next();
  };
};

void wxToolBar3::OnLeftClick( wxToolBar3Tool *t )
{
  owner->Command( t->command );
};

/*#
 * Drawing of inverse rectangle while placing toolbar
 */

/*!
 * StartInvRect() starts the drawing of the XOR-rectangle for toolbar-placing.
 */
void wxToolBar3::StartInvRect(void)
{
  wxScreenDC dc;
  dc.StartDrawingOnTop();
}

/*!
 * EndInvRect() ends the drawing of the XOR-rectangle for toolbar-placing.
 */
void wxToolBar3::EndInvRect(void)
{
  // ungrab server, drawing done
  wxScreenDC dc;
  dc.EndDrawingOnTop();
}

/*!
 * DrawInvRect(x, y) draws the XOR rectangle for toolbar-palcing.
 * /p x, y: mouse position relative to window
 */
void wxToolBar3::DrawInvRect( int absMouseX, int absMouseY )
{
  int x = absMouseX - relMouseX;
  int y = absMouseY - relMouseY;
  ClientToScreen( &x, &y );
  int w = GetLength()+5;
  int h = wxTOOLBARHEIGHT;
  owner->MakeDockableRect( x, y, w, h );
  wxScreenDC dc;
  wxColour color( 255, 255, 255 );
  wxPen pen( color, 2, wxSOLID );
  dc.SetPen( &pen );
  dc.SetBrush( wxTRANSPARENT_BRUSH );
  dc.SetLogicalFunction( wxXOR );
  dc.DrawRectangle( x, y, w, h );
};

void wxToolBar3::OnEventWhileWaiting( int x, int y, wxMouseEvent& event )
{
  wxToolBar3Tool *tool = GetToolByPosition( x, y );

  if (!event.IsButton()) // no mousebuttons were pressed -> (re-)start the timer
  {
	 if (tool != NULL)
	 {
		currentHint = tool;
		timerRunning = TRUE;
		timer->Start( 500, TRUE );
	 };
	 return;
  };

  if ((tool == NULL) && (event.LeftDClick())) // double changes state immediately
  {
	 if (position == wxTBFloating)
		owner->MakeFixed( this, wxTBTop );
	 else
		owner->MakeFloating( this, 200, 200, 70, 40 );
	 state = wxWaiting;
	 return;
  };

  if (!event.LeftDown()) return;

  if (tool == NULL) // mousebutton pressed, but not on a tool -> move ToolBar3
  {
	 CaptureMouse();
	 relMouseX = x;
	 relMouseY = y;
	 StartInvRect();
	 DrawInvRect( x, y );
	 oldMouseX = x;
	 oldMouseY = y;
	 state = wxMoving;
	 wxCursor *cursor = new wxCursor(wxCURSOR_SIZING);
	 SetCursor( cursor );
	 return;
  };

  if (tool->isEnabled) // mousebutton pressed on a tool -> push down ToolBar3
  {
	 tool->Draw( this, FALSE );
	 currentTool = tool;
	 CaptureMouse();
	 state = wxPressed;
  };
};

void wxToolBar3::OnEventWhilePressed( int x, int y, wxMouseEvent& event )
{
  wxToolBar3Tool *tool = GetToolByPosition( x, y );

  if (event.LeftUp())
  {
	 state = wxNothing;
	 currentTool->Draw( this, TRUE );
	 ReleaseMouse();
	 if (tool == currentTool) OnLeftClick( tool );
	 state = wxWaiting;
  }
  else
  {
	 if (tool == currentTool)
		currentTool->Draw( this, FALSE );
	 else
		currentTool->Draw( this, TRUE );
  };
};

void wxToolBar3::OnEventWhileMoving( int x, int y, wxMouseEvent& event )
{
  DrawInvRect( oldMouseX, oldMouseY );

  if (event.LeftUp())
  {
	 ReleaseMouse();
         SetCursor( wxSTANDARD_CURSOR );
	 int finalX = x - relMouseX;
	 int finalY = y - relMouseY;
	 ClientToScreen( &finalX, &finalY );
	 int finalW = GetLength();
	 int finalH = wxTOOLBARHEIGHT;
	 wxToolBar3Pos newPosition = owner->MakeDockableRect( finalX, finalY, finalW, finalH );
	 if (newPosition == wxTBFloating)
	 {
		if (position == wxTBFloating)
		  GetParent()->SetSize( finalX-1, finalY-12, -1, -1, wxSIZE_USE_EXISTING );
		else
		  owner->MakeFloating( this, finalX, finalY, finalW, finalH );
	 }
	 else
	 {
		if (newPosition != position)
		{
		  if (position == wxTBFloating)
		  {
			 owner->MakeFixed( this, newPosition );
		  }
		  else
		  {
			 position = newPosition;
			 owner->OnSize( 0, 0 );
		  };
		};
	 };
	 state = wxWaiting;
	 EndInvRect(); 
	 return;
  };
  DrawInvRect( x, y );
  oldMouseX = x;
  oldMouseY = y;
};

void wxToolBar3::OnEvent(wxMouseEvent& event)
{
  if (event.GetEventClass() != wxTYPE_MOUSE_EVENT) return;

  if (state == wxHinting)
  {
	 EraseHint();
	 state = wxWaiting;
  };
  if (timerRunning)
  {
	 timer->Stop();
	 timerRunning = FALSE;
  };

  float x, y;
  event.Position(&x, &y);     // returns > 65000 for negative values...
  if (x > 20000) x -= 65535;
  if (y > 20000) y -= 65535;

  switch (state)
  {
	 case wxWaiting: OnEventWhileWaiting( (int)x, (int)y, event ); return;
	 case wxMoving: OnEventWhileMoving( (int)x, (int)y, event ); return;
	 case wxPressed: OnEventWhilePressed( (int)x, (int)y, event ); return;
	 default: break;
  };
};

void wxToolBar3::DoDrawRect( wxDC *dc, int x, int y, int w, int h, Bool up )
{
  if (up)
  {
    dc->SetBrush( wxTRANSPARENT_BRUSH );

    dc->SetPen( wxWHITE_PEN );
    dc->DrawRectangle( x, y, w-1, 0 );            // top  (outer)
    dc->DrawRectangle( x, y, 0, h );              // left (outer)

    dc->SetPen( wxMEDIUM_GREY_PEN );
    dc->DrawRectangle( x+w, y, 0, h );            // right (outer)
    dc->DrawRectangle( x, y+h, w, 0 );            // bottom (outer)

    dc->SetPen( wxLIGHT_GREY_PEN );
    dc->DrawRectangle( x+1, y+1, w-2, h-2 );      // (total inner rect)
  }
  else
  {
    dc->SetBrush( wxTRANSPARENT_BRUSH );

    dc->SetPen( wxMEDIUM_GREY_PEN );
    dc->DrawRectangle( x, y, w-1, 0 );            // top  (outer)
    dc->DrawRectangle( x, y, 0, h );              // left (outer)

    dc->SetPen( wxWHITE_PEN );
    dc->DrawRectangle( x+w, y, 0, h );            // right (outer)
    dc->DrawRectangle( x, y+h, w, 0 );            // bottom (outer)
  };
};

void wxToolBar3::DrawTool( wxToolBar3Tool *t, wxDC *dc )
{
  memDC.SelectObject( t->bitmap );
  DoDrawRect( dc, t->xpos, t->ypos, t->w+3, t->h+3, t->isUp );
  if (t->isUp)
	 dc->Blit( t->xpos+2, t->ypos+2, t->w, t->h, &memDC, 0, 0 );
  else
	 dc->Blit( t->xpos+3, t->ypos+3, t->w, t->h, &memDC, 0, 0 );
  memDC.SelectObject( NULL );
};
void wxToolBar3::OnPaint()
{
  wxDC *dc = GetDC();
  dc->BeginDrawing();

  int width;
  int height;
  GetSize( &width, &height);
  dc->SetPen( wxLIGHT_GREY_PEN);
  dc->SetBrush( wxLIGHT_GREY_BRUSH);
  dc->DrawRectangle( 0, 0, width, height );
  /*
  if (position == wxTBTop)
  {
	 dc->SetPen( wxWHITE_PEN);
	 dc->DrawLine( 2, 2, width-2, 2 );
	 dc->SetPen( wxMEDIUM_GREY_PEN);
	 dc->DrawLine( 2, 1, width-2, 1 );
  };
  int len = GetLength();
  if ((position == wxTBTop) || (position == wxTBBottom))
  {
	 dc->SetPen( wxWHITE_PEN);
	 dc->DrawLine( len+5, 4, len+5, height-2 );
	 dc->SetPen( wxMEDIUM_GREY_PEN);
	 dc->DrawLine( len+6, 4, len+6, height-2 );
  };
  */

  wxNode *node = tools.First();
  while (node)
  {
	 wxToolBar3Tool *t = (wxToolBar3Tool*) node->Data();
	 DrawTool( t, dc );
	 node = node->Next();
  };

  dc->EndDrawing();
};

void wxToolBar3::DoNotifyHint()
{
  timer->Stop();
  timerRunning = FALSE;
/* Windows..
  POINT pt;
  LPPOINT lpt = &pt;
  GetCursorPos( lpt );
  int x = pt.x;
  int y = pt.y;
*/
/* X */
  XMotionEvent xev;
  Window root,child;
  XQueryPointer( wxGetDisplay(), DefaultRootWindow(wxGetDisplay()), 
    &root, &child, &(xev.x_root), &(xev.y_root), &(xev.x), &(xev.y), &(xev.state) );
  int x = xev.x;
  int y = xev.y;
/* end X */  
  ScreenToClient( &x, &y );
  if (currentHint->IsIn( x, y))
  {
	 DrawHint();
	 state = wxHinting;
  }
  else
	 currentHint = NULL;
};

void wxToolBar3::DrawHint(void)
{
  //  cerr << "DrawHint()" << endl;
  wxScreenDC dc;
  wxRectangle hint;
  hint.x = currentHint->xpos+15;
  hint.y = currentHint->ypos+currentHint->h+3;
  ClientToScreen(&hint.x, &hint.y);
  float w = 0;
  float h = 0;
  dc.SetFont( wxSWISS_FONT );
  dc.GetTextExtent( currentHint->hint.GetData(), &w, &h );
  hint.width  = (int)w + 8;
  hint.height = (int)h + 4;
  dc.StartDrawingOnTop(&hint);
  dc.SetPen( wxBLACK_PEN );
  dc.SetBrush( wxWHITE_BRUSH );
  dc.DrawRectangle(hint.x, hint.y, hint.width-1, hint.height-1 );
  dc.DrawText( currentHint->hint.GetData(), hint.x+3, hint.y+2 );
};

void wxToolBar3::EraseHint(void)
{
  //  cerr << "EraseHint()" << endl;
  wxScreenDC dc;
  dc.EndDrawingOnTop();
};

void wxToolBar3::MakeToolsForAnotherToolBar3( wxToolBar3 *tb )
{
  wxNode *node = tools.First();
  while (node)
  {
	 wxToolBar3Tool *t = (wxToolBar3Tool*) node->Data();
	 tb->AppendTool( t->bitmap, t->xpos, t->ypos, t->command,
						  t->isState, t->hint.GetData() );
	 node = node->Next();
  };
};

int wxToolBar3::GetLength(void)
{
  wxNode *node = tools.Last();
  if (node)
  {
	 wxToolBar3Tool *t = (wxToolBar3Tool*) node->Data();
	 return t->xpos + t->w + 5;
  }
  else
    return 15;
};

void wxToolBar3::SetSizeAndOwnersSize(void)
{
  int len = GetLength()+5;
  SetSize( 0, 0, len, wxTOOLBARHEIGHT, wxSIZE_USE_EXISTING );
  if (position == wxTBFloating)
  {
	 wxFrame *parent = (wxFrame*) GetParent();
	 int parentW;
	 int parentH;
	 parent->GetSize( &parentW, &parentH );
	 int clientW;
	 int clientH;
	 parent->GetClientSize( &clientW, &clientH );
	 int restX = parentW-clientW;
	 int restY = parentH-clientH;
	 parent->SetSize( -1, -1, len+restX, wxTOOLBARHEIGHT+restY, wxSIZE_USE_EXISTING );
  };
};

// -----------------------------------------
// wxDecoratedFrame
// -----------------------------------------

IMPLEMENT_DYNAMIC_CLASS(wxDecoratedFrame,wxFrame);

wxDecoratedFrame::wxDecoratedFrame( wxFrame *frame, char *title,
  int x, int y, int w, int h ) :
  wxFrame( frame, title, x, y, w, h )
{
};

void wxDecoratedFrame::OnSize( int w, int h )
{
  int x = 0;
  int y = 0;
  GetClientSize( &w, &h );
  wxNode *node = fixedToolBar3s.First();
  while (node)
  {
	 wxToolBar3 *tb = (wxToolBar3*) node->Data();
	 switch (tb->position)
	 {
		case wxTBTop:
		{
		  tb->SetSize( x, y, w, wxTOOLBARHEIGHT );
		  y += wxTOOLBARHEIGHT;
		  h -= wxTOOLBARHEIGHT;
		  break;
		};
		case wxTBBottom:
		{
		  tb->SetSize( x, y+h-wxTOOLBARHEIGHT, w, wxTOOLBARHEIGHT );
		  h -= wxTOOLBARHEIGHT;
		  break;
		};
		default: break;
	 };
	 node = node->Next();
  };
  OnDecoratedSize( x, y, w, h );
  Refresh( FALSE );
};

void wxDecoratedFrame::OnDecoratedSize( int WXUNUSED(x), int WXUNUSED(y), int WXUNUSED(w), int WXUNUSED(h) )
{
};

wxToolBar3Pos wxDecoratedFrame::MakeDockableRect( int &x, int &y, int &WXUNUSED(w), int &WXUNUSED(h) )
{
  int myX,myY,myW,myH;
  myX = 0;
  myY = 0;
  ClientToScreen( &myX, &myY );
  GetClientSize( &myW, &myH );
  if ((abs(x-myX) < 15) && (abs(y-myY) < 15))
  {
	 x = myX;
	 y = myY;
	 return wxTBTop;
  };
  if ((abs(x-myX) < 15) && (abs(y-myH-myY+wxTOOLBARHEIGHT) < 15))
  {
	 x = myX;
	 y = myY+myH-wxTOOLBARHEIGHT;
	 return wxTBBottom;
  };
  return wxTBFloating;
};

void wxDecoratedFrame::MakeFloating( wxToolBar3 *tb, int x, int y, int w, int WXUNUSED(h) )
{
  fixedToolBar3s.DeleteObject( tb );
  wxFrame *f = new wxFrame( this, "Tools", x-5, y-10, w+10, 40 );
  wxToolBar3 *ToolBar3 = new wxToolBar3( f, this, wxTBFloating );
  ToolBar3->SetSizeAndOwnersSize();
  tb->MakeToolsForAnotherToolBar3( ToolBar3 );
  f->Show( TRUE );
  delete tb;
  floatingToolBar3s.Append( ToolBar3 );
  OnSize( 0, 0 );
};

void wxDecoratedFrame::MakeFixed( wxToolBar3 *tb, wxToolBar3Pos pos )
{
  floatingToolBar3s.DeleteObject( tb );
  wxToolBar3 *ToolBar3 = new wxToolBar3( this, this, pos );
  tb->MakeToolsForAnotherToolBar3( ToolBar3 );
  delete tb->GetParent();
  fixedToolBar3s.Append( ToolBar3 );
  OnSize( 0, 0 );
};

wxToolBar3 *wxDecoratedFrame::NewToolBar3( wxToolBar3Pos pos, int x, int y )
{
  wxToolBar3 *toolBar;
  if (pos == wxTBFloating)
  {
	 wxFrame *f = new wxFrame( this, "Tools", x, y, 100, wxTOOLBARHEIGHT );
	 toolBar = new wxToolBar3( f, this, wxTBFloating );
	 toolBar->SetSizeAndOwnersSize();
	 f->Show( TRUE );
	 floatingToolBar3s.Append( toolBar );
  }
  else
  {
	 toolBar = new wxToolBar3( this, this, pos );
	 fixedToolBar3s.Append( toolBar );
  };
  return toolBar;
};

void wxDecoratedFrame::Enable( int id, Bool flag )
{
  wxNode *node = fixedToolBar3s.First();
  while (node)
  {
	 wxToolBar3 *t = (wxToolBar3*)node->Data();
	 t->Enable( id, flag );
	 node = node->Next();
  };
  node = floatingToolBar3s.First();
  while (node)
  {
	 wxToolBar3 *t = (wxToolBar3*)node->Data();
	 t->Enable( id, flag );
	 node = node->Next();
  };
  wxMenuBar *menu = GetMenuBar();
  if (menu) menu->Enable( id, flag );
};

/*
 * Local Variables:
 * mode: c++
 * tab-width:4
 * comment-column:40
 * c-basic-offset:2
 * End:
 */
