/*
 * File:       WIN_toolbar3
 * Purpose:    Toolbar
 * Author:     Robert Roebling
 * Created:    1997
 * Copyright:  (c) Robert Roebling 1997
 */

#ifndef ToolBar3_h
#define ToolBar3_h

#ifdef __GNUG__
#pragma interface
#endif

class wxToolBar3Timer;
class wxToolBar3Tool;
class wxToolBar3;
class wxDecoratedFrame;

// -----------------------------------------
// wxToolBar3Timer
// -----------------------------------------

class wxToolBar3Timer: public wxTimer
{
  DECLARE_DYNAMIC_CLASS(wxToolBar3Timer);
  
  public:
	 wxToolBar3 *owner;

	 wxToolBar3Timer( wxToolBar3 *o );
	 wxToolBar3Timer() : wxTimer() {};
	 virtual void Notify(void);
};

// -----------------------------------------
// wxToolBar3Tool
// -----------------------------------------

class wxToolBar3Tool: public wxObject
{
  DECLARE_DYNAMIC_CLASS(wxToolBar3Tool);
  
  public:

	  int       xpos,ypos;
	  int       w,h;
	  int       command;
	  wxBitmap *bitmap;
	  Bool      isState;
	  Bool      isOn;
	  Bool      isEnabled;
	  Bool      isUp;
	  wxString  hint;

	  wxToolBar3Tool( wxBitmap *b, int x, int y, int c, Bool s, char *text );
	  wxToolBar3Tool() : wxObject() {};
	  ~wxToolBar3Tool(void);
	  Bool IsIn( int x, int y );
	  void Draw( wxToolBar3 *toolBar, Bool up );
};

// -----------------------------------------
// wxToolBar3
// -----------------------------------------

enum wxToolBar3State { wxWaiting, wxPressed, wxHinting, wxMoving, wxNothing };
enum wxToolBar3Pos { wxTBTop, wxTBBottom, wxTBFloating };

class wxToolBar3: public wxPanel
{
  DECLARE_DYNAMIC_CLASS(wxToolBar3);

  friend wxDecoratedFrame;
  friend wxToolBar3Timer;

  private:

	 wxToolBar3Pos      position;
	 wxToolBar3State    state;
	 wxList            tools;
	 wxMemoryDC        memDC;
	 wxToolBar3Tool    *currentTool,*currentHint;
	 wxDecoratedFrame *owner;
	 wxToolBar3Timer   *timer;
	 Bool              timerRunning;
	 int               relMouseX,relMouseY;
	 int               oldMouseX,oldMouseY;
	 int               hintX,hintY,hintW,hintH;
	 wxCursor         *oldCursor;

  public: 
	 wxToolBar3() : wxPanel() {};
	 wxToolBar3( wxWindow *parent, wxDecoratedFrame* o, wxToolBar3Pos p );
	 ~wxToolBar3();
	 wxToolBar3Tool *GetToolByCommand( int c );
	 wxToolBar3Tool *GetToolByPosition( int x, int y );
	 void Enable( int id, Bool flag );
	 void OnEventWhileMoving( int x, int y, wxMouseEvent& event );
	 void OnEventWhilePressed( int x, int y, wxMouseEvent& event );
	 void OnEventWhileWaiting( int x, int y, wxMouseEvent& event );
	 void OnEvent( wxMouseEvent& event );
	 void OnLeftClick( wxToolBar3Tool *t );
         void DoDrawRect( wxDC *dc, int x, int y, int w, int h, Bool up );
	 void DrawTool( wxToolBar3Tool *t, wxDC *dc );
	 void OnPaint();
    // Hint
	 void DoNotifyHint(void);
	 void DrawHint(void);
	 void EraseHint(void);
     // XOR rectangle
     void StartInvRect(void);
	 void DrawInvRect( int absMouseX, int absMouseY );
     void EndInvRect(void);
	 void MakeToolsForAnotherToolBar3( wxToolBar3 *tb );
	 int GetLength(void);
	 void SetSizeAndOwnersSize(void);

  public:

	 void AppendTool( wxBitmap *b, int x, int y, int c, Bool s, char *h );
};

class wxDecoratedFrame: public wxFrame
{
  DECLARE_DYNAMIC_CLASS(wxDecoratedFrame);

  friend wxToolBar3;

  private:

	 wxList      fixedToolBar3s,floatingToolBar3s;

	 wxToolBar3Pos MakeDockableRect( int &x, int &y, int &w, int &h );
	 void MakeFloating( wxToolBar3 *tb, int x, int y, int w, int h );
	 void MakeFixed( wxToolBar3 *tb, wxToolBar3Pos pos );

  public:
	 wxDecoratedFrame() : wxFrame() {};
	 wxDecoratedFrame( wxFrame *frame, char *title, int x, int y, int w, int h );
	 operator = ( wxDecoratedFrame &WXUNUSED(df) ) { return 0; };
	 wxToolBar3 *NewToolBar3( wxToolBar3Pos pos, int x = 200, int y = 100 );
	 void Enable( int id, Bool flag );
	 virtual void OnSize( int w, int h );
	 virtual void OnDecoratedSize( int x, int y, int w, int h );
};

/*
 * Local Variables:
 * mode: c++
 * tab-width:4
 * comment-column:40
 * c-basic-offset:2
 * End:
 */

#endif
