/*
 * Copyright: (C) 1997, GNU (Robert Roebling)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef __GNUG__
#pragma implementation "WIN_treectrl.h"
#endif

#define Uses_wxPanel
#define Uses_wxCanvas
#define Uses_wxString
#include "wx.h"

#include <math.h>
#include "WIN_treectrl.h"

//-----------------------------------------------------------------------------
//  wxVirtualBranch
//-----------------------------------------------------------------------------

IMPLEMENT_DYNAMIC_CLASS(wxVirtualBranch,wxObject);

wxVirtualBranch::wxVirtualBranch( void )
{  
  name = "anchor";
  ID = 0;
  level = 0;
  hasChildren = TRUE;
  parent = NULL;
  crossX = 0;
  crossY = 0;
//  children.DeleteContents( 1 );  memory leak, but uncommenting gives some error
};

wxVirtualBranch::wxVirtualBranch( Bool children, char *n, int id  )
{
  hasChildren = children;
  name = n;
  ID = id;
  level = 0;
  parent = NULL;
  crossX = 0;
  crossY = 0;
//  children.DeleteContents( TRUE ); 
};

char *wxVirtualBranch::GetName( void )
{
   return name;
};

int wxVirtualBranch::GetID( void )
{
   return ID;
};

int wxVirtualBranch::GetLevel( void )
{
  return level;
};

void wxVirtualBranch::AddChild( wxVirtualBranch *s )
{
  s->SetLevelAndParent( level+1, this );
  children.Append( s );
};

Bool wxVirtualBranch::HasChildren( void )
{
  return hasChildren;
};

void wxVirtualBranch::SetHasChildren( Bool has )
{
  hasChildren = has;
};

Bool wxVirtualBranch::HasPlus( void )
{
  return (hasChildren && (children.Number() == 0));
};

Bool wxVirtualBranch::HasMinus( void )
{
  return (hasChildren && (children.Number() > 0));
};

int wxVirtualBranch::GetYPos( void )
{
  return (int)crossY;
};

void wxVirtualBranch::KillChildren( void )
{
  children.Clear();
};

int wxVirtualBranch::GetNumVisibleChildren( void )
{
  return children.Number();
};

int wxVirtualBranch::GetNumVisibleDescendants( void )
{
  int res = 1;
  wxNode *node = children.First();
  while (node)
  {
    wxVirtualBranch *b = (wxVirtualBranch*)(node->Data());
    res += b->GetNumVisibleDescendants();
    node = node->Next();
  };
  return res;
};

wxVirtualBranch *wxVirtualBranch::GetChild( int idx )
{
  wxNode *node = children.Nth( idx );
  return (wxVirtualBranch*)(node->Data());
};

wxVirtualBranch *wxVirtualBranch::GetParent( void )
{
  return parent;
};

wxVirtualBranch *wxVirtualBranch::GetNext( void )
{
  if (!parent) return NULL;
  wxNode *node = parent->children.Member( this )->Next();
  if (node)
    return (wxVirtualBranch*)(node->Data());
  else
    return parent->GetNext();
};

wxVirtualBranch *wxVirtualBranch::GetPrevious( void )
{
  if (!parent) return NULL;
  wxNode *node = parent->children.Member( this )->Previous();
  if (node)
    return (wxVirtualBranch*)(node->Data());
  else
    return parent->GetPrevious();
};

wxVirtualBranch *wxVirtualBranch::GetNextOnSameLevel( void )
{
  if (!parent) return NULL;
  wxNode *node = parent->children.Member( this )->Next();
  if (node) return (wxVirtualBranch*)(node->Data());
  wxVirtualBranch *granny = parent->parent;
  if (granny)
  {
    node = granny->GetChildrenList()->Member( parent )->Next();
    if (node) return (wxVirtualBranch*)(node->Data());
  };
  return NULL;
};

wxVirtualBranch *wxVirtualBranch::GetPreviousOnSameLevel( void )
{
  if (!parent) return NULL;
  wxNode *node = parent->children.Member( this )->Previous();
  if (node) return (wxVirtualBranch*)(node->Data());
  return NULL;
};

int wxVirtualBranch::CompareTo( wxVirtualBranch *WXUNUSED(b) )
{
  return 0;
};

int comp_func( const void **arg1, const void **arg2 )
{
  wxVirtualBranch *b1 = *(wxVirtualBranch**)arg1;
  wxVirtualBranch *b2 = *(wxVirtualBranch**)arg2;
  return b1->CompareTo( b2 );
};

void wxVirtualBranch::SortChildren( void )
{
  children.Sort( comp_func );
};

// proteced

void wxVirtualBranch::SetLevelAndParent( int l, wxVirtualBranch *p )
{
  parent = p;
  level = l;
};

wxVirtualBranch *wxVirtualBranch::GetHit( float x, float y, hitType &hit)
{
  wxNode *node = children.First();
  while (node)
  {
    wxVirtualBranch *child = (wxVirtualBranch*)(node->Data());
    if (child->HasChildren())
    {
      if (child->InCross( x, y )) 
      {
        if (child->GetNumVisibleChildren() == 0) 
          hit = wxHIT_PLUS; else hit = wxHIT_MINUS;
        return child;
      };
    }; 
    if (child->InBody( x, y ))
    {
      hit = wxHIT;
      return child;
    };
    wxVirtualBranch *res = child->GetHit( x, y, hit );
    if (res) return res;
    node = node->Next();
  };
  return NULL;
};

void wxVirtualBranch::SetCross( float x, float y )
{
  crossX = x;
  crossY = y;
};

void wxVirtualBranch::GetCross( float &x, float &y )
{
  x = crossX;
  y = crossY;
};

Bool wxVirtualBranch::InCross( float x, float y )
{
  if (fabs(crossX - x) > 4) return FALSE;
  if (fabs(crossY - y) > 4) return FALSE;
  return TRUE;
};

Bool wxVirtualBranch::InBody( float x, float y )
{
  if ((crossX+18 - x) > 1) return FALSE;
  if (fabs(crossY - y) > 5) return FALSE;
  return TRUE;
};

//-----------------------------------------------------------------------------
//  wxVirtualTree
//-----------------------------------------------------------------------------

IMPLEMENT_DYNAMIC_CLASS(wxVirtualTree,wxCanvas);

wxVirtualTree::wxVirtualTree( void )
{
};

wxVirtualTree::wxVirtualTree( wxPanel *p, int x, int y, int width, int height,
	     long style, Constdata char *name ) : 
  wxCanvas( p, x, y, width, height, style, name )
{	  
  lineSpace = 16;
  textHeight = 12;
  current = &anchor;

  wxColour *hl = new wxColour( 208, 180, 176 );  // deallocate ?
  highlightBrush = new wxBrush( (*hl), wxSOLID );
  myFont = ((wxPanel*)GetParent())->GetButtonFont();
  
  hasPaintedBefore = FALSE;
  xScroll = 0;  
  yScroll = 10;  
  SetScrollbars( 0, yScroll, 0, 100 / yScroll, 0, 20 ); // anything
};

void wxVirtualTree::HighlightCurrent( wxCanvasDC *dc, Bool hilite )
{
  dc->SetPen( wxTRANSPARENT_PEN );
  if (hilite)
    dc->SetBrush( highlightBrush );
  else
    dc->SetBrush( wxWHITE_BRUSH );
  dc->SetFont( myFont );
  float x = 0;
  float y = 0;
  current->GetCross( x, y );
  x += 18;
  y -= textHeight/2;
  float w = 0;
  float h = 0;
  dc->GetTextExtent( current->GetName(), &w, &h );
  dc->DrawRectangle( x-1, y-1, w+2, h+2 );
  dc->DrawText( current->GetName(), x, y );
};

void wxVirtualTree::HighlightCurrent( Bool hilite )
{
  wxCanvasDC *dc = GetDC();
  wxPen *oldPen = dc->GetPen();
  wxBrush *oldBrush = dc->GetBrush();
  wxFont* oldFont = dc->GetFont();
  dc->BeginDrawing();

  HighlightCurrent( dc, hilite );

  dc->EndDrawing();
  dc->SetPen( oldPen );
  dc->SetBrush( oldBrush );
  dc->SetFont( oldFont );
};

void wxVirtualTree::DrawLevel( wxVirtualBranch *b, wxCanvasDC *dc, int level, int &y )
{
  dc->SetFont( myFont );
  int horizX = level*15+10;
  int oldY = y;
  wxNode *node = b->GetChildrenList()->First();
  while (node)
  {
    dc->SetPen( dottedPen );
    wxVirtualBranch *child = (wxVirtualBranch*)(node->Data());
    child->SetCross( horizX+15, y );
    int startX,endX; 
    if ((node->Previous()) || (level != 0)) 
      startX = horizX; else startX = horizX-10;
    if (child->HasChildren()) 
      endX = horizX+10; else endX = horizX+30;
    DrawLine( startX, y, endX, y );
    if (!node->Next())
    {
      if (level != 0) oldY -= (lineSpace-5);
      DrawLine( horizX, oldY, horizX, y );
    }
    if (child->HasChildren())
    {
      dc->DrawLine( horizX+20, y, horizX+30, y );
      dc->SetPen( wxGREY_PEN );
      dc->DrawRectangle( horizX+10, y-4, 10, 8 );
      dc->SetPen( wxBLACK_PEN );
      dc->DrawLine( horizX+13, y, horizX+17, y );
      if (child->HasPlus())
        dc->DrawLine( horizX+15, y-2, horizX+15, y+2 );
    };
    dc->DrawText( child->GetName(), horizX+33, y-textHeight/2 );
    y += lineSpace;
    if (child->GetNumVisibleChildren() > 0) 
      DrawLevel( child, dc, level+1, y );
    node = node->Next();
  };
};

void wxVirtualTree::OnPaint( void )
{	  
  if ((current == (&anchor)) && (anchor.children.Number()))
  {
    wxNode *node = anchor.children.First();
    current = (wxVirtualBranch*) (node->Data());
  };

  wxCanvasDC *dc = GetDC();
  wxPen *oldPen = dc->GetPen();
  wxBrush *oldBrush = dc->GetBrush();
  wxFont* oldFont = dc->GetFont();
  dc->BeginDrawing();
  dottedPen = new wxPen(  "grey", 1, wxDOT );
  
  float w,h;
  GetTextExtent( "H", &w, &h );
  textHeight = (int)h;
  lineSpace = textHeight+4;
  
  int y = lineSpace / 2 + 2;
  DrawLevel( &anchor, dc, 0, y );

  HighlightCurrent( dc, TRUE );
  
  dc->EndDrawing();
  dc->SetPen( oldPen );
  dc->SetBrush( oldBrush );
  dc->SetFont( oldFont );
};

void wxVirtualTree::Display( void )
{
  wxCanvasDC *dc = GetDC();
  dc->SetFont( myFont );

  float w,h;
  GetTextExtent( "H", &w, &h );
  textHeight = (int)h;
  lineSpace = textHeight+4;
  
  int entireLen = anchor.GetNumVisibleDescendants() * lineSpace;
  xScroll = 0;
  yScroll = lineSpace;
  SetScrollbars( 0, yScroll, 0, entireLen / yScroll, 0, 20 );
};

void wxVirtualTree::MoveToFocus( void )
{
  if (!current) return;
  int y = current->GetYPos();
  int w_p = 0;
  int h_p = 0;
  GetClientSize( &w_p, &h_p );
  int y_s = yScroll * GetScrollPos( wxVERTICAL );
  if (y-5 < y_s) 
  {
    SetScrollPos( wxVERTICAL, (y-10)/yScroll );
    return;
  };
  if (y+5 > y_s+h_p)
  {
    SetScrollPos( wxVERTICAL, (y+18-h_p)/yScroll );
    return;
  };
};

void wxVirtualTree::OnEvent( wxMouseEvent &event )
{
  float x = 0;
  float y = 0;
  event.Position( &x, &y );
  if (event.LeftDown())
  {
    hitType h = wxNO_HIT;
    wxVirtualBranch *b = anchor.GetHit( x, y, h );
    if (!b) return;
    Bool changedHL = (b != current); 
    switch (h)
    {
      case wxHIT_PLUS:
        {
          current = b;
          int s = GetScrollPos( wxVERTICAL );
          ExpandCurrent();
          Display();
          SetScrollPos( wxVERTICAL, s );
        };
        return;
      case wxHIT_MINUS:
        {
          current = b;
          int s = GetScrollPos( wxVERTICAL );
          CollapseCurrent();
          Display();
          SetScrollPos( wxVERTICAL, s );
        };
        return;
      case wxHIT:
        if (changedHL) HighlightCurrent( FALSE );        
        current = b;
        if (changedHL) HighlightCurrent( TRUE );
        current->Activated();
        return;
      default:
        return;
    };
  };
};

void wxVirtualTree::OnChar( wxKeyEvent &event )
{	  
  wxVirtualBranch *b;
  switch (event.KeyCode())
  {
    case ' ':
    case WXK_RETURN:
    case WXK_EXECUTE:
      current->Activated();
      if (current->HasPlus())
      {
        int s = GetScrollPos( wxVERTICAL );
        ExpandCurrent();
        Display();
        SetScrollPos( wxVERTICAL, s );
      };
      break;
    case '-':
    case WXK_SUBTRACT:
      if (current->HasMinus())
      {
        int s = GetScrollPos( wxVERTICAL );
        CollapseCurrent();
        Display();
        SetScrollPos( wxVERTICAL, s );
      };
      break;
    case '+':
    case WXK_ADD:
      if (current->HasPlus())
      {
        int s = GetScrollPos( wxVERTICAL );
        ExpandCurrent();
        Display();
        SetScrollPos( wxVERTICAL, s );
      };
      break;
    case WXK_UP:
      b = current->GetPreviousOnSameLevel();
      if (!b)
      {
        b = current->GetParent();
        if (b == &anchor) b = NULL;
      };
      if (b)
      {
        HighlightCurrent( FALSE );
        current = b;
        HighlightCurrent( TRUE );
        MoveToFocus();
      };
      break;
    case WXK_DOWN:
      if (current->HasMinus())
        b = current->GetChild( 0 );
      else
        b = current->GetNextOnSameLevel();
      if (b)
      {
        HighlightCurrent( FALSE );
        current = b;
        MoveToFocus();
        HighlightCurrent( TRUE );
      };
      break;
    case WXK_LEFT:
      b = current->GetParent();
      if (b != &anchor)
      {
        HighlightCurrent( FALSE );
        current = b;
        HighlightCurrent( TRUE );
        MoveToFocus();
      }; 
      break;
    case WXK_RIGHT:
      if (current->HasMinus())
      {
        b = current->GetChild( 0 );
        HighlightCurrent( FALSE );
        current = b;
        HighlightCurrent( TRUE );
        MoveToFocus();
      }; 
      break;
    default:
      break;
  };
};

wxVirtualBranch *wxVirtualTree::GetFirst( void )
{	  
  wxNode *node = children.First();
  return (wxVirtualBranch*)(node->Data());
};

wxVirtualBranch *wxVirtualTree::GetNext( wxVirtualBranch *b )
{
  return b->GetNext();
};

wxVirtualBranch *wxVirtualTree::GetHit( int x, int y )
{
  hitType h = wxNO_HIT;
  return anchor.GetHit( x, y, h );
};

wxVirtualBranch *wxVirtualTree::GetCurrent( void )
{
  return current;
};

void wxVirtualTree::AddToCurrent( wxVirtualBranch *b )
{
  if (current)
  {
    b->SetLevelAndParent( current->GetLevel()+1, current );
    current->AddChild( b );
  };
};

void wxVirtualTree::CollapseAll( void )
{	  
  // soon to come, maybe
};

void wxVirtualTree::ExpandAll( void )
{
};

void wxVirtualTree::CollapseCurrent( void )
{	  
  current->KillChildren();
};

void wxVirtualTree::ExpandCurrent( void )
{
  current->Expand();	  
};

