/*
 *
 * Copyright: (C) 1997, GNU (Robert Roebling)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef TreeCtrl_h
#define TreeCtrl_h

#ifdef __GNUG__
#pragma interface
#endif

#include "WIN_treectrl.h"

//-----------------------------------------------------------------------------
// classes
//-----------------------------------------------------------------------------

class wxVirtualBranch;
class wxVirtualTree;

//-----------------------------------------------------------------------------
//  constants
//-----------------------------------------------------------------------------

enum hitType  { wxNO_HIT, wxHIT_PLUS, wxHIT_MINUS, wxHIT };

//-----------------------------------------------------------------------------
//  wxVirtualBranch
//-----------------------------------------------------------------------------

class wxVirtualBranch : public wxObject
{
  DECLARE_DYNAMIC_CLASS(wxVirtualBranch)

  public:
    wxVirtualBranch( void );
    wxVirtualBranch( Bool children, char *n = "error", int id = 0 );
    virtual void Expand( void ) {};
    virtual void Highlight( void ) {};
    virtual void Activated( void ) {};

//  private:
    char   *name;
    int	    ID;

    virtual char *GetName( void );
    int GetID( void );
    int GetLevel( void );
    void AddChild( wxVirtualBranch *s );
    void SetHasChildren( Bool has );
    Bool HasChildren( void );
    Bool HasPlus( void );
    Bool HasMinus( void );
    int GetYPos( void );
    void KillChildren( void );
    int GetNumVisibleChildren( void );
    int GetNumVisibleDescendants( void );
    wxVirtualBranch *GetChild( int idx );
    wxVirtualBranch *GetParent( void );
    wxVirtualBranch *GetPrevious( void );
    wxVirtualBranch *GetNext( void );
    wxVirtualBranch *GetNextOnSameLevel( void );
    wxVirtualBranch *GetPreviousOnSameLevel( void );
    virtual int CompareTo( wxVirtualBranch *b );
    void SortChildren( void );

  protected:
    int               level;
    wxList            children;
    Bool              hasChildren;
    wxVirtualBranch  *parent;
    float             crossX,crossY;

    void SetLevelAndParent( int l, wxVirtualBranch *p );
    wxVirtualBranch *GetHit( float x, float y, hitType &hit );
    wxList *GetChildrenList( void ) { return &children; };
    void SetCross( float x, float y );
    void GetCross( float &x, float &y );
    Bool InCross( float x, float y );
    Bool InBody( float x, float y );

  friend wxVirtualTree;  
};

//-----------------------------------------------------------------------------
//  wxVirtualTree
//-----------------------------------------------------------------------------

class wxVirtualTree : public wxCanvas
{
  DECLARE_DYNAMIC_CLASS(wxVirtualTree)
  private:
    wxVirtualBranch   anchor;
    wxVirtualBranch  *current;
    int               lineSpace;
    int               textHeight;
    wxPen            *dottedPen;
    wxBrush          *highlightBrush;
    wxFont           *myFont;
    int               xScroll,yScroll;
    Bool              hasPaintedBefore;
    
  public:
    wxVirtualTree( void ); 
    wxVirtualTree( wxPanel *p, 
	     int x=-1, int y=-1, int width=-1, int height=-1,
	     long style=0, Constdata char *name= "wxVirtualTree" );
    void HighlightCurrent( wxCanvasDC *dc, Bool hilite );
    void HighlightCurrent( Bool hilite );
    void DrawLevel( wxVirtualBranch *b, wxCanvasDC *dc, int level, int &y );
    void OnPaint( void );
    void Display( void );
    void MoveToFocus( void );
    void OnEvent( wxMouseEvent &event );
    void OnChar( wxKeyEvent &event );
    wxVirtualBranch *GetFirst( void );
    wxVirtualBranch *GetNext( wxVirtualBranch *b );
    wxVirtualBranch *GetCurrent( void );
    wxVirtualBranch *GetHit( int x, int y );
    void AddToCurrent( wxVirtualBranch *b );
    void CollapseAll( void );
    void ExpandAll( void );
    void CollapseCurrent( void );
    void ExpandCurrent( void );
};

#endif // TreeCtrl_h
